#!/usr/bin/env python3
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
from pyromaths.outils import Geometrie as geo
import random, math
from pyromaths.outils.Affichage import decimaux
from pyromaths.outils.Geometrie import cotation, cotation_h
from pyromaths.ex import LegacyExercise
from decimal import Decimal

# trigo en degré
tan = lambda z:math.tan(math.radians(z))
cos = lambda z:math.cos(math.radians(z))
sin = lambda z:math.sin(math.radians(z))
def shuffle_nom(polygone):
    """renvoie un nom aléatoire du polygone"""
    n = len(polygone)
    polygone = polygone + polygone + polygone
    debut = n + random.randrange(n)
    sens = [-1, 1][random.randrange(2)]
    nom = ""
    for i in range(debut, debut + n * sens, sens):
        nom += polygone[i]
    return nom

def _exo_triangle(test=False):
    questions = [quest_equilateral,
           quest_LAL,
           quest_ALA,
           quest_AAL,
           quest_isocele_angbase,
           quest_isocele_angprincipal,
           quest_rectangle_hypo_cote,
           quest_rectangle_hypo_angle,
           ]
    exo = ["\\exercice",
         "\\begin{enumerate}"]
    cor = ["\\exercice*",
         "\\begin{enumerate}"]
    cor.append("\\definecolor{enonce}{rgb}{0.11,0.56,0.98}")
    cor.append("\\definecolor{calcul}{rgb}{0.13,0.54,0.13}")
    cor.append(u"\\psset{MarkAngleRadius=0.6,PointSymbol=none}")
    if test:  # toutes les constructions en test
        for quest in questions:
            quest(exo, cor)
    else:  # Construction choisies au hasard
        random.shuffle(questions)
        for i in range(4):
            questions[i](exo, cor)

    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo, cor

class exo_triangle(LegacyExercise):
    """Construction de triangles"""

    tags = ["Cinquième"]
    function = _exo_triangle


def quest_equilateral(exo, cor):
    A, B, C = geo.choix_points(3)
    nom = shuffle_nom([A, B, C])
    c = 0.1 * random.randint(40, 70)  # longueur AB
    angBAC = angABC = 60
    exo.append(_(u"\\item Trace un triangle $%s$ équilatéral de côté $\\unit[%s]{cm}$.\\par") % (nom, decimaux(c)))
    cor.append(_(u"\\item Trace un triangle $%s$ équilatéral de côté $\\unit[%s]{cm}$.\\par") % (nom, decimaux(c)))
    x_C = c * tan(angABC) / (tan(angABC) + tan(angBAC))
    y_C = x_C * tan(angBAC)
    cor.append(u"\\begin{pspicture}(-1,-1)(%.3f,%.3f)" % (c + 1, y_C + 1))
    cor.append(u"\\pstTriangle(0,0){%s}(%.3f,0){%s}(%.3f,%.3f){%s}" % (A, c, B, x_C, y_C, C))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (C, A))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (B, C))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (A, B))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[C_1]" % (A, C))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[C_2]" % (A, C))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{C_2}{C_1}" % (A))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[C_3]" % (B, C))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[C_4]" % (B, C))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{C_4}{C_3}" % (B))
    cor.append(cotation((0, 0), (c, 0), decimaux(c), couleur="enonce"))
    cor.append(u"\\end{pspicture}")

def quest_LAL(exo, cor):
    """on donne un angle et les longueurs de ses deux côtés"""
    """            angBAC et      AB=c et AC=b"""
    A, B, C = geo.choix_points(3)
    nom = shuffle_nom([A, B, C])
    c = 0.1 * random.randint(40, 70)  # longueur AB
    b = 0.1 * random.randint(20, 100)  # longueur BC
    angBAC = 3 * random.randint(7, 50)  # BAC mesure entre 21° et 150°

    exo.append(_(u"\\item Trace un triangle $%s$ tel que $%s%s=\\unit[%s]{cm}$, $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$")
               % (nom, A, B, decimaux(c), A, C, decimaux(b), B, A, C, angBAC))
    cor.append(_(u"\\item Trace un triangle $%s$ tel que $%s%s=\\unit[%s]{cm}$, $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, A, B, decimaux(c), A, C, decimaux(b), B, A, C, angBAC))
    cor.append(u"\\begin{pspicture}(%.3f,%.3f)(%.3f,%.3f)" % (min(0, b * cos(angBAC)) - 0.4, -1, max(b, b * cos(angBAC)) + 0.4, b * sin(angBAC) + 1))
    cor.append(u"\\pstTriangle(0,0){%s}(%.3f;%.3f){%s}(%.3f,0){%s}" % (A, b, angBAC, C, c, B))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}" % (B, A, C, angBAC))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}" % (A, C))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[C_1]" % (A, C))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[C_2]" % (A, C))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{C_2}{C_1}" % (A))
    cor.append(cotation((0, 0), (c, 0), decimaux(c), couleur="enonce"))
    if angBAC < 111:
        x_C, y_C = b * cos(angBAC), b * sin(angBAC)
        cor.append(cotation_h((0, 0), (x_C, y_C), decimaux(b), couleur="enonce"))
    else:
        x_C, y_C = b * cos(angBAC), b * sin(angBAC)
        cor.append(cotation((x_C, y_C), (0, 0), decimaux(b), couleur="enonce"))
    cor.append(u"\\end{pspicture}")

def quest_ALA(exo, cor):
    """on donne deux angles et la longueur du côté commun"""
    """ angBAC et angABC et AB=c"""
    A, B, C = geo.choix_points(3)
    nom = shuffle_nom([A, B, C])
    c = 0.1 * random.randint(40, 70)  # longueur AB
    angBAC = 5 * random.randint(4, 12)
    angABC = 5 * random.randint(4, 12)
    exo.append(_(u"\\item Trace un triangle $%s$ tel que $%s%s=\\unit[%s]{cm}$,  $\\widehat{%s%s%s}=%s\\degres$ et $\\widehat{%s%s%s}=%s\\degres$")
               % (nom, A, B, decimaux(c), B, A, C, angBAC, A, B, C, angABC))
    cor.append(_(u"\\item Trace un triangle $%s$ tel que $%s%s=\\unit[%s]{cm}$,  $\\widehat{%s%s%s}=%s\\degres$ et $\\widehat{%s%s%s}=%s\\degres$\\par")
               % (nom, A, B, decimaux(c), B, A, C, angBAC, A, B, C, angABC))
    x_C = c * tan(angABC) / (tan(angABC) + tan(angBAC))
    y_C = x_C * tan(angBAC)
    cor.append(u"\\begin{pspicture}(-0.4,-1)(%.3f,%.3f)" % (c + 1, y_C + 1))
    cor.append(u"\\pstTriangle(0,0){%s}(%.3f,0){%s}(%.3f,%.3f){%s}" % (A, c, B, x_C, y_C, C))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}" % (A, C, B, C))
    cor.append(cotation((0, 0), (c, 0), decimaux(c), couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}" % (B, A, C, angBAC))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}" % (C, B, A, angABC))
    cor.append(u"\\end{pspicture}")

def quest_AAL(exo, cor):
    """on donne deux angles et la longueur d'un côté non commun aux deux angles"""
    """ angBAC et angACB et AB=c"""
    A, B, C = geo.choix_points(3)
    nom = shuffle_nom([A, B, C])
    angBAC = 3 * random.randint(10, 24)  # entre 30° et 72°
    angACB = 3 * random.randint(10, 24)  # entre 30° et 72°
    angABC = 180 - angBAC - angACB
    ABmax = int(35 * (tan(angABC) + tan(angBAC)) / (tan(angBAC) * tan(angABC)))  # donne une hauteur inférieur à 35*0.2=7 cm
    c = 0.2 * random.randint(20, max(20, ABmax))  # longueur AB

    exo.append(_(u"\\item Trace un triangle $%s$ tel que $%s%s=\\unit[%s]{cm}$,  $\\widehat{%s%s%s}=%s\\degres$ et $\\widehat{%s%s%s}=%s\\degres$")
               % (nom, A, B, decimaux(c), B, A, C, angBAC, A, C, B, angACB))
    cor.append(_(u"\\item Trace un triangle $%s$ tel que $%s%s=\\unit[%s]{cm}$,  $\\widehat{%s%s%s}=%s\\degres$ et $\\widehat{%s%s%s}=%s\\degres$\\par")
               % (nom, A, B, decimaux(c), B, A, C, angBAC, A, C, B, angACB))
    cor.append(_(u"On doit d'abord calculer la mesure de $\\widehat{%s%s%s}$.\\\\")
               % (A, B, C))
    cor.append(_(u"Or la somme des trois angles d'un triangle est égale à 180\\degres donc $\\widehat{%s%s%s}=180\\degres-%s\\degres-%s\\degres=%s\\degres$.\\par")
               % (A, B, C, angBAC, angACB, angABC))
    x_C = c * tan(angABC) / (tan(angABC) + tan(angBAC))
    y_C = x_C * tan(angBAC)
    cor.append(u"\\begin{pspicture}(-1,-1)(%.3f,%.3f)" % (c + 1, y_C + 1))
    cor.append(u"\\pstTriangle(0,0){%s}(%.3f,0){%s}(%.3f,%.3f){%s}" % (A, c, B, x_C, y_C, C))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}" % (A, C, B, C))
    cor.append(u"\\pstMarkAngle[linecolor=calcul]{%s}{%s}{%s}{\\color{calcul}%s\\degres}" % (C, B, A, angABC))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}" % (A, C, B, angACB))
    cor.append(cotation((0, 0), (c, 0), decimaux(c), couleur="enonce"))
    cor.append(u"\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}" % (B, A, C, angBAC))
    cor.append(u"\\end{pspicture}")

def quest_isocele_angbase(exo, cor):
    """on donne ABC isocele en C, AB=c et angBAC"""
    A, B, C = geo.choix_points(3)
    nom = shuffle_nom([A, B, C])
    angABC = angBAC = random.randint(20, 70)
    maxc = max(20, int(35 / tan(angBAC)))  # longueur c maximale pour que la hauteur soit 7 cm =0.2*35
    minc = min(20, int(35 / tan(angBAC)))
    c = 0.2 * random.randint(minc, min(maxc, 50))  # longueur AB
    # Calcul pour tracer
    x_C = c // 2
    y_C = x_C * tan(angBAC)

    exo.append(_(u"\\item Tracer un triangle $%s$ isocèle en $%s$ tel que $%s%s=\\unit[%s]{cm}$,  $\\widehat{%s%s%s}=%s\\degres$.")
               % (nom, C, A, B, decimaux(c), B, A, C, angBAC))
    cor.append(_(u"\\item Tracer un triangle $%s$ isocèle en $%s$ tel que $%s%s=\\unit[%s]{cm}$,  $\\widehat{%s%s%s}=%s\\degres$. \\par")
               % (nom, C, A, B, decimaux(c), B, A, C, angBAC))
    cor.append(_(u"Comme $%s%s%s$ est un triangle isocèle en $%s$, on sait que les angles adjacents à la base sont de même mesure \
donc $\\widehat{%s%s%s}=\\widehat{%s%s%s}=%s\\degres$.\\par") % (A, B, C, C, A, B, C, B, A, C, angBAC))
    cor.append(u"\\begin{pspicture}(-1,-1)(%.3f,%.3f)" % (c + 1, y_C + 1))
    cor.append(u"\\pstTriangle(0,0){%s}(%.3f,0){%s}(%.3f,%.3f){%s}" % (A, c, B, x_C, y_C, C))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}" % (A, C))
    cor.append("\\pstLineAB[linestyle=none]{%s}{%s} \\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (A, C))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}" % (B, C))
    cor.append(u"\\pstLineAB[linestyle=none]{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (B, C))
    cor.append(cotation((0, 0), (c, 0), decimaux(c), couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}" % (B, A, C, angBAC))
    cor.append(u"\\color{calcul}\\pstMarkAngle[linecolor=calcul]{%s}{%s}{%s}{%s\\degres}" % (C, B, A, angABC))
    cor.append(u"\\end{pspicture}")

def quest_isocele_angprincipal(exo, cor):
    """on donne ABC isocele en C, AB=c et angACB"""
    A, B, C = geo.choix_points(3)
    nom = shuffle_nom([A, B, C])
    angACB = 2 * random.randint(20, 60)  # ACB est pair entre 40° et 120°
    angBAC = angABC = (180 - angACB) / 2
    maxc = max(20, int(35 / tan(angBAC)))  # longueur c maximale pour que la hauteur soit 7 cm =0.2*35
    c = 0.2 * random.randint(20, min(maxc, 35))  # longueur AB
    # Calcul pour tracer
    x_C = c // 2
    y_C = x_C * tan(angBAC)

    exo.append(_(u"\\item Tracer un triangle $%s$ isocèle en $%s$ tel que $%s%s=\\unit[%s]{cm}$,  $\\widehat{%s%s%s}=%s\\degres$.")
               % (nom, C, A, B, decimaux(c), A, C, B, angACB))
    cor.append(_(u"\\item Tracer un triangle $%s$ isocèle en $%s$ tel que $%s%s=\\unit[%s]{cm}$,  $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, C, A, B, decimaux(c), A, C, B, angACB))
    cor.append(_(u"Comme $%s%s%s$ est un triangle isocèle en $%s$, on sait que les angles adjacents à la base sont de même mesure \
donc $\\widehat{%s%s%s}=\\widehat{%s%s%s}$.\\par") % (A, B, C, C, A, B, C, B, A, C))
    cor.append(_(u"De plus, on sait que la somme des mesures des trois angles d'un triangle est égale à 180\\degres \\\\ \
donc $\\widehat{%s%s%s}=\\widehat{%s%s%s}=(180\\degres-%s\\degres)\\div 2=%s\\degres$. \\par")
               % (B, A, C, A, B, C, angACB, angBAC))
    cor.append(u"\\begin{pspicture}(-1,-1)(%.3f,%.3f)" % (c + 1, y_C + 1))
    cor.append(u"\\pstTriangle(0,0){%s}(%.3f,0){%s}(%.3f,%.3f){%s}" % (A, c, B, x_C, y_C, C))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}" % (A, C))
    cor.append("\\pstLineAB[linestyle=none]{%s}{%s} \\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (A, C))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}" % (B, C))
    cor.append("\\pstLineAB[linestyle=none]{%s}{%s} \\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (B, C))
    cor.append(cotation((0, 0), (c, 0), decimaux(c), couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce,Mark=MarkHash]{%s}{%s}{%s}{%s\\degres}" % (A, C, B, angACB))
    cor.append(u"\\color{calcul}\\pstMarkAngle[linecolor=calcul]{%s}{%s}{%s}{%s\\degres}" % (B, A, C, angBAC))
    cor.append(u"\\pstMarkAngle[linecolor=calcul]{%s}{%s}{%s}{%s\\degres}" % (C, B, A, angABC))
    cor.append(u"\\end{pspicture}")

def quest_rectangle_hypo_angle(exo, cor):
    """on donne un triangle ABC rectangle en C et l'hypotenuse AB et l'angle BAC"""
    A, B, C = geo.choix_points(3)
    nom = shuffle_nom([A, B, C])
    c = 0.2 * random.randint(20, 35)  # longueur AB
    angBAC = 3 * random.randint(7, 23)  # un angle mesurant entre 21° et 69°
    angABC = 90 - angBAC
    # angACB=90
    # Calcul pour tracer
    x_C = c * tan(angABC) / (tan(angABC) + tan(angBAC))
    y_C = x_C * tan(angBAC)

    exo.append(_(u"\\item Tracer un triangle $%s$ rectangle en $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, C, A, B, decimaux(c), B, A, C, angBAC))
    cor.append(_(u"\\item Tracer un triangle $%s$ rectangle en $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, C, A, B, decimaux(c), B, A, C, angBAC))
# #    cor.append("\\begin{multicols}{2}")
    cor.append(_(u"On sait que dans un triangle rectangle, les deux angles aigus sont complémentaires \\\\ \
donc $\widehat{%s%s%s}=90\\degres-%s\\degres=%s\\degres$.\\par") % (B, A, C, angBAC, angABC))

    cor.append("\\figureadroite{")
    cor.append(u"\\begin{pspicture}(-0.4,-1)(%.3f,%.3f)" % (c + 0.4, y_C + 1))
    cor.append(u"\\pstTriangle(0,0){%s}(%.3f,0){%s}(%.3f,%.3f){%s}" % (A, c, B, x_C, y_C, C))
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}" % (A, C, B))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}" % (A, C, B, C))
    cor.append(cotation((0, 0), (c, 0), decimaux(c), couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}" % (B, A, C, angBAC))
    cor.append(u"\\color{calcul}\\pstMarkAngle[linecolor=calcul]{%s}{%s}{%s}{%s\\degres}" % (C, B, A, angABC))
    cor.append(u"\\end{pspicture}}{")
    cor.append("\\begin{enumerate}")
    cor.append(_(u"\\item On trace le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;") % (A, B, decimaux(c)))
    cor.append(_(u"\\item puis la demi-droite $[%s%s)$ en traçant l'angle~$\widehat{%s%s%s}$ ;") % (A, C, B, A, C))
    cor.append(_(u"\\item puis la demi-droite $[%s%s)$ en traçant l'angle~$\widehat{%s%s%s}$ ;") % (B, C, A, B, C))
# #    cor.append(u"\\item enfin on vérifie les trois {\\color{enonce}conditions de l'énoncé}.")
    cor.append("\\end{enumerate}\n")
# #    cor.append("\\end{multicols}")
    cor.append("}")

def quest_rectangle_hypo_cote(exo, cor):
    """on donne un triangle ABC rectangle en B et l'hypotenuse AC et le coté AB"""
    A, B, C = geo.choix_points(3)
    nom = shuffle_nom([A, B, C])
    c = 0.2 * random.randint(20, 35)  # longueur AB
    b = 0.1 * random.randint(int(10 * (c)) + 1, 100)  # longueur AC
    angABC = 90
    # calcul pour tracer
    angBAC = math.degrees(math.acos(c / b))
    x_C = c * tan(angABC) / (tan(angABC) + tan(angBAC))
    y_C = x_C * tan(angBAC)

    exo.append(_(u"\\item Tracer un triangle $%s$ rectangle en $%s$ tel que $%s%s=\\unit[%s]{cm}$,  $%s%s=\\unit[%s]{cm}$.\\par")
               % (nom, B, A, B, decimaux(c), A, C, decimaux(b)))
    cor.append(_(u"\\item Tracer un triangle $%s$ rectangle en $%s$ tel que $%s%s=\\unit[%s]{cm}$, $%s%s=\\unit[%s]{cm}$.\\par")
               % (nom, B, A, B, decimaux(c), A, C, decimaux(b)))
# #    cor.append("\\begin{multicols}{2}")
    cor.append("\\figureadroite{")
    cor.append(u"\\begin{pspicture}(-0.4,-1)(%.3f,%.3f)" % (c + 0.4, y_C + 1))
    cor.append(u"\\pstTriangle(0,0){%s}(%.3f,0){%s}(%.3f,%.3f){%s}" % (A, c, B, x_C, y_C, C))
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}" % (C, B, A))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[C_1]" % (A, C))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[C_2]" % (A, C))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{C_2}{C_1}" % (A))
    cor.append(u"\\pstLineAB{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}" % (A, B, B, C))
    cor.append(cotation((0, 0), (c, 0), decimaux(c), couleur="enonce"))
    cor.append(cotation_h((0, 0), (x_C, y_C), decimaux(b), couleur="enonce"))
    cor.append(u"\\end{pspicture}}{")
    cor.append(u"\\begin{enumerate}")
    cor.append(_(u"\\item On trace le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;") % (A, B, decimaux(c)))
    cor.append(_(u"\\item puis on trace l'angle droit $\\widehat{%s%s%s}$ ;") % (A, B, C))
    cor.append(_(u"\\item enfin, on reporte au compas la longueur \\mbox{$%s%s=\\unit[%s]{cm}$} à partir de $%s$.")
                % (A, C, decimaux(b), A))
    cor.append("\\end{enumerate}}")
# #    cor.append("\\end{multicols}")


#################################################################################################
#   Les construction sont faites avec [AB] horizontale (ou avec la diagonale [AC] horizontale)
#       D-------C
#      /       /
#     /       /
#    A-------B
#   A est toujours l'origine : A(0,0)
#
#   AB est la longueur AB : type(float) en cm
#   angBAC est la mesure de l'angle BAC : type(int) en degrés
#   x_C,y_C sont les coordonnées cartésiennes du point C
#  les coordonnées polaires sont notées (5 ; 60)
#
###################################################################################################

def _exo_quadrilatere(test=False):

    exo = ["\\exercice",
         "\\begin{enumerate}"]
    cor = ["\\exercice*",
         "\\begin{enumerate}"]
    cor.append("\\definecolor{enonce}{rgb}{0.11,0.56,0.98}")
    # couleur pour reporter les informations de la consigne
    cor.append("\\definecolor{calcul}{rgb}{0.13,0.54,0.13}")
    # couleur pour reporter des informations déduites par raisonnement ou calcul
    cor.append(u"\\psset{MarkAngleRadius=0.7,PointSymbol=none,dotscale=2}")
    # MarkAngleRadius = rayon de l'arc marquant les angles
    # PointSymbol=none les points ne sont pas tracés
    # dotscale=2, grossit la  croix si PointSymbol=x

    # liste des questions possibles
    questions = [quest_rectangle_diag,
           quest_rectangle_angle,
           quest_rectangle_angle_diag,
           quest_rectangle_diag_angle,
           quest_parallelogramme_CCA,
           quest_parallelogramme_CDA,
           quest_parallelogramme_DDA,
           quest_losange_DD,
           quest_losange_CD,
           quest_losange_CDbis,
           quest_losange_CC,
           carre_diag]

    if test:  # toutes les constructions en test
        for quest in questions:
            quest(exo, cor)
    else:
        # on choisit un parallélogramme, un losange et un rectangle
        questions[random.randrange(4)](exo, cor)
        questions[random.randrange(4, 7)](exo, cor)
        questions[random.randrange(7, 11)](exo, cor)
        # questions[11], le carré n'est jamais proposé
    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo, cor

class exo_quadrilatere(LegacyExercise):
    """Construction de parallélogrammes"""

    tags = ["Cinquième"]
    function = _exo_quadrilatere

################################################################

####### RECTANGLES #############

def quest_rectangle_diag(exo, cor):
    """on donne un rectangle ABCD avec un côté AB et une diagonale AC"""
    A, B, C, D = geo.choix_points(4)
    nom = shuffle_nom([A, B, C, D])
    L = random.randint(40, 60)  # AB mesure entre 4cm et 7cm, tracé horizontalement
    Diag = 0.1 * random.randint(L + 10, 70)  # +1.1 pour éviter les problèmes d'arrondi
    L = 0.1 * L
    # Calcul pour tracer
    angBAC = math.degrees(math.acos(L / Diag))
    x_C = L
    y_C = x_C * tan(angBAC)

    exo.append(_(u"\\item Tracer un rectangle $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $%s%s=\\unit[%s]{cm}$.\\par")
               % (nom, A, B, decimaux(L), A, C, decimaux(Diag)))
    cor.append(_(u"\\item Tracer un rectangle $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $%s%s=\\unit[%s]{cm}$.\\par")
               % (nom, A, B, decimaux(L), A, C, decimaux(Diag)))
    # figure
    cor.append("\\figureadroite{")
    cor.append(u"\\begin{pspicture}(-0.4,-1)(%.3f,%.3f)" % (L + 0.4, y_C + 1))
    cor.append(u"\\pstTriangle(0,0){%s}(%.3f,0){%s}(%.3f,%.3f){%s}" % (A, L, B, x_C, y_C, C))
    cor.append(u"\\pstGeonode[PosAngle=135](0,%.3f){%s}" % (y_C, D))
    # codage
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}" % (D, C, B))
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}" % (B, A, D))
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}" % (C, B, A))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[C_1]" % (A, C))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[C_2]" % (A, C))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{C_2}{C_1}" % (A))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}" % (A, D, C, D, A, B, B, C))
    cor.append(cotation((0, 0), (L, 0), decimaux(L), couleur="enonce"))
    cor.append(cotation_h((0, 0), (x_C, y_C), decimaux(Diag), couleur="enonce"))
    cor.append(u"\\end{pspicture}}{")
    # Programme de construction
    cor.append(u"\\begin{enumerate}")
    cor.append(_(u"\\item On trace le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;") % (A, B, decimaux(L)))
    cor.append(_(u"\\item puis on trace l'angle droit $\\widehat{%s%s%s}$ ;") % (A, B, C))
    cor.append(_(u"\\item On reporte au compas la longueur $%s%s=\\unit[%s]{cm}$ à partir de $%s$ ;") % (A, C, decimaux(Diag), A))
    cor.append(_(u"\\item On trace enfin les angles droits en $%s$ et en $%s$ pour placer le point $%s$.") % (A, C, D))
    cor.append("\\end{enumerate}}")


def quest_rectangle_angle(exo, cor):
    """On donne un rectangle ABCD avec le côté AB et l'angle BAC"""
    A, B, C, D = geo.choix_points(4)
    nom = shuffle_nom([A, B, C, D])
    L = random.randint(40, 60)  # AB mesure entre 4cm et 7cm, tracé horizontalement
    angBAC = random.randint(25, 65)
    L = 0.1 * L
    # Calcul pour tracer
    x_C = L
    y_C = x_C * tan(angBAC)

    exo.append(_(u"\\item Tracer un rectangle $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, A, B, decimaux(L), B, A, C, angBAC))
    cor.append(_(u"\\item Tracer un rectangle $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, A, B, decimaux(L), B, A, C, angBAC))

    cor.append("\\figureadroite{")
    cor.append(u"\\begin{pspicture}(-0.4,-1)(%.3f,%.3f)" % (L + 0.4, y_C + 1))
    cor.append(u"\\pstTriangle(0,0){%s}(%.3f,0){%s}(%.3f,%.3f){%s}" % (A, L, B, x_C, y_C, C))
    cor.append(u"\\pstGeonode[PosAngle=135](0,%.3f){%s}" % (y_C, D))
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}" % (D, C, B))
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}" % (B, A, D))
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}" % (C, B, A))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}"
               % (A, D, C, D, A, B, B, C, A, C))
    cor.append(cotation((0, 0), (L, 0), decimaux(L), couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}" % (B, A, C, angBAC))
    cor.append(u"\\end{pspicture}}{")
    # Programme de construction
    cor.append(u"\\begin{enumerate}")
    cor.append(_(u"\\item On trace le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;") % (A, B, decimaux(L)))
    cor.append(_(u"\\item puis on trace l'angle droit $\\widehat{%s%s%s}$ ;") % (A, B, C))
    cor.append(_(u"\\item la demi-droite $[%s%s)$ en mesurant $\\widehat{%s%s%s}=%s\\degres$.") % (A, C, B, A, C, angBAC))
    cor.append(_(u"\\item On trace enfin les angles droit en $%s$ et en $%s$ pour placer le point $%s$.") % (A, C, D))
    cor.append("\\end{enumerate}}")

def quest_rectangle_angle_diag(exo, cor):
    """On donne un rectangle ABCD de centre E, la diagonale AC et l'angle AEB"""
    A, B, C, D, E = geo.choix_points(5)
    nom = shuffle_nom([A, B, C, D])
    angAEB = 2 * random.randint(20, 70)
    Diag = 0.2 * random.randint(25, 45)
    # calcul pour tracer
    angBAC = (180 - angAEB) // 2
    L = Diag * cos(angBAC)
    x_C = L
    y_C = x_C * tan(angBAC)

    exo.append(_(u"\\item Tracer un rectangle $%s$ de centre $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, E, A, C, decimaux(Diag), A, E, B, angAEB))
    cor.append(_(u"\\item Tracer un rectangle $%s$ de centre $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, E, A, C, decimaux(Diag), A, E, B, angAEB))
# #    cor.append("\\begin{multicols}{2}")
    # Programme de construction
    cor.append("\\figureadroite{")
# #    cor.append("\columnbreak")
    cor.append(u"\\begin{pspicture}(-0.4,-1)(%.3f,%.3f)" % (L + 0.4, y_C + 1))
    cor.append(u"\\pstTriangle(0,0){%s}(%.3f,0){%s}(%.3f,%.3f){%s}" % (A, L, B, x_C, y_C, C))
    cor.append(u"\\pstGeonode[PosAngle={135,0}](0,%.3f){%s}(%.3f,%.3f){%s}" % (y_C, D, x_C / 2, y_C / 2, E))
    cor.append(u"\\pstLineAB{%s}{%s}" % (C, D))
    cor.append(u"\\pstLineAB{%s}{%s}" % (A, D))
    cor.append(u"\\pstLineAB[nodesep=-1]{%s}{%s}" % (B, D))
    # codage
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHashh}" % (E, A))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHashh}" % (E, B))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHashh}" % (E, C))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHashh}" % (E, D))
    cor.append(u"\\psarc[linecolor=calcul](%.3f,%.3f){%s}{%s}{%s} " % (x_C / 2, y_C / 2, Diag // 2, angBAC - 7, angBAC + 7))
    cor.append(u"\\psarc[linecolor=calcul](%.3f,%.3f){%s}{%s}{%s} " % (x_C / 2, y_C / 2, Diag // 2, 180 - angBAC - 7, 180 - angBAC + 7))
    cor.append(u"\\psarc[linecolor=calcul](%.3f,%.3f){%s}{%s}{%s} " % (x_C / 2, y_C / 2, Diag // 2, -angBAC - 7, -angBAC + 7))
    cor.append(u"\\psarc[linecolor=calcul](%.3f,%.3f){%s}{%s}{%s} " % (x_C / 2, y_C / 2, Diag // 2, 180 + angBAC - 7, 180 + angBAC + 7))
    cor.append("\\pcline[linestyle=none](0,0)(%.3f,%.3f)\\aput*{:U}{\\color{enonce}\\unit[%s]{cm}}" % (L, y_C, decimaux(Diag)))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}" % (A, E, B, angAEB))
    cor.append(u"\\end{pspicture}}{")
    cor.append(u"\\begin{enumerate}")
    cor.append(_(u"\\item On trace le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;") % (A, C, decimaux(Diag)))
    cor.append(_(u"\\item le centre du rectangle est le milieu des diagonales donc $%s$ est le milieu de $[%s%s]$ ;") % (E, A, C))
    cor.append(_(u"\\item On trace la diagonale $(%s%s)$ passant par $%s$ en mesurant \\mbox{$\\widehat{%s%s%s}=%s\\degres$} ;")
               % (B, D, E, A, E, B, angAEB))
    cor.append(_(u"\\item Comme les diagonales du rectangle sont de même longueur, on reporte les longueurs $%s%s=%s%s=\\unit[%s]{cm}$.")
               % (E, D, E, B, decimaux(Diag // 2)))
# #    cor.append(_(u"\\item On trace enfin les angles droits en $%s$ et en $%s$ pour palcer le point $%s$.")%(A,C,D))
    cor.append("\\end{enumerate}}")
# #    cor.append("\\end{multicols}")

def quest_rectangle_diag_angle(exo, cor):
    """On donne un rectangle ABCD la diagonale AC et l'angle BAC"""
    A, B, C, D = geo.choix_points(4)
    nom = shuffle_nom([A, B, C, D])
    angBAC = random.randint(25, 65)
    Diag = 0.1 * random.randint(50, 80)
    # Calcul
    L = Diag * cos(angBAC)
    x_C = L
    y_C = x_C * tan(angBAC)

    exo.append(_(u"\\item Tracer un rectangle $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, A, C, decimaux(Diag), B, A, C, angBAC))
    cor.append(_(u"\\item Tracer un rectangle $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, A, C, decimaux(Diag), B, A, C, angBAC))

# #    cor.append("\\begin{multicols}{2}")
    cor.append("\\figureadroite{")
# #    cor.append("\columnbreak")
    # figure
    cor.append(u"\\begin{pspicture}(-0.4,-1)(%.3f,%.3f)" % (L + 0.4, y_C + 1))
    cor.append(u"\\pstTriangle(0,0){%s}(%.3f,0){%s}(%.3f,%.3f){%s}" % (A, L, B, x_C, y_C, C))
    cor.append(u"\\pstGeonode[PosAngle=135](0,%.3f){%s}" % (y_C, D))
    # codage
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}" % (D, C, B))
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}" % (B, A, D))
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}" % (C, B, A))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB[nodesep=-1]{%s}{%s}"
               % (A, D, C, D, A, B, B, C))
    cor.append(cotation_h((0, 0), (L, y_C), decimaux(Diag), couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}" % (B, A, C, angBAC))
    cor.append(u"\\end{pspicture}}{")
    # Programme de construction
    cor.append(u"\\begin{enumerate}")
    cor.append(_(u"\\item On tracer le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;") % (A, C, decimaux(Diag)))
    cor.append(_(u"\\item la demi-droite $[%s%s)$ en mesurant \\mbox{$\\widehat{%s%s%s}=%s\\degres$} ;") % (A, B, B, A, C, angBAC))
    cor.append(_(u"\\item puis la perpendiculaire à $[%s%s)$ passant par~$%s$ ;") % (A, B, C))
    cor.append(_(u"\\item On tracer enfin les angles droits en $%s$ et en $%s$ pour placer le point~$%s$.") % (A, C, D))
    cor.append("\\end{enumerate}}")
# #    cor.append("\\end{multicols}")

################## PARALLÉLOGRAMMES QUELCONQUES ###########################

def quest_parallelogramme_CCA(exo, cor):
    """On donne un parallélogramme avec la longueur de deux côtés et un angle."""
    # 3 choix d'angle : CC ou un autre angle CC ou un angle Côté Diagonale
    A, B, C, D = geo.choix_points(4)
    nom = shuffle_nom([A, B, C, D])
    AB = 0.1 * random.randint(40, 60)  # AB mesure entre 4cm et 7cm, tracé horizontalement
    AD = 0.1 * random.randint(40, 60)  # AD mesure entre 4cm et 7cm, tracé horizontalement
    angBAD = random.randint(25, 65)
    # Pour tracer
    x_C = AB + AD * cos(angBAD)
    y_C = AD * sin(angBAD)

    exo.append(_(u"\\item Tracer un parallélogramme $%s$ tel que $%s%s=\\unit[%s]{cm}$, $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, A, B, decimaux(AB), D, A, decimaux(AD), B, A, D, angBAD))
    cor.append(_(u"\\item Tracer un parallélogramme $%s$ tel que $%s%s=\\unit[%s]{cm}$, $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, A, B, decimaux(AB), D, A, decimaux(AD), B, A, D, angBAD))
    cor.append(u"\\begin{enumerate}")
    cor.append(_(u"\\item On trace le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;") % (A, B, decimaux(AB)))
    cor.append(_(u"\\item On mesure l'angle  $\\widehat{%s%s%s}=%s\\degres$ puis on place le point~$%s$ ;") % (B, A, D, angBAD, D))
    cor.append(_(u"\\item enfin on reporte les longueurs $%s%s=%s%s$ et $%s%s=%s%s$ pour place le point~$%s$.")
               % (D, C, A, B, B, C, A, D, C))
    cor.append("\\end{enumerate}\n")
    cor.append(u"\\begin{pspicture}(-0.4,-1)(%.3f,%.3f)" % (max(AB, x_C) + 0.4, y_C + 1))
    cor.append(u"\\pstGeonode[PosAngle={-135,-45,45,135}](0,0){%s}(%.3f,0){%s}(%.3f,%.3f){%s}(%.3f;%.3f){%s}" % (A, AB, B, x_C, y_C, C, AD, angBAD, D))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB{%s}{%s}\\pstLineAB{%s}{%s}\\pstLineAB{%s}{%s}"
               % (A, D, C, D, A, B, B, C))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[C_1]" % (D, C))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[C_2]" % (D, C))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{C_2}{C_1}" % (D))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[D_1]" % (A, D))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[D_2]" % (A, D))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{D_2}{D_1}" % (A))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[C_3]" % (B, C))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[C_4]" % (B, C))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{C_4}{C_3}" % (B))
    cor.append(cotation_h((0, 0), (x_C - AB, y_C), decimaux(AD), couleur="enonce"))
    cor.append(cotation((0, 0), (AB, 0), decimaux(AB), couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}" % (B, A, D, angBAD))
    cor.append(u"\\pstLineAB[linestyle=none]{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHashh}" % (A, D))
    cor.append(u"\\pstLineAB[linestyle=none]{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHashh}" % (B, C))
    cor.append(u"\\pstLineAB[linestyle=none]{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHash}" % (A, B))
    cor.append(u"\\pstLineAB[linestyle=none]{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHash}" % (D, C))
    cor.append(u"\\end{pspicture}")

def quest_parallelogramme_CDA(exo, cor):
    """On donne un parallélogramme avec la longueur d'un côté et une diagonale et l'angle entre ces deux segments"""

    A, B, C, D = geo.choix_points(4)
    nom = shuffle_nom([A, B, C, D])
    AB = 0.1 * random.randint(40, 60)  # AB mesure entre 4cm et 7cm, tracé horizontalement
    AC = 0.1 * random.randint(40, 70)  # AC mesure entre 4cm et 7cm, tracé horizontalement
    angBAC = random.randint(25, 65)
    # Calcul pour tracer
    x_C = round(AC * cos(angBAC), 4)  # round() évite une écriture scientifique si x_D=2.4e-15, non reconnue par PSTricks
    y_D = y_C = AC * sin(angBAC)
    x_D = round(x_C - AB, 4)

    exo.append(_(u"\\item Tracer un parallélogramme $%s$ tel que $%s%s=\\unit[%s]{cm}$, $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, A, B, decimaux(AB), C, A, decimaux(AC), B, A, C, angBAC))
    cor.append(_(u"\\item Tracer un parallélogramme $%s$ tel que $%s%s=\\unit[%s]{cm}$, $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, A, B, decimaux(AB), C, A, decimaux(AC), B, A, C, angBAC))
    # Programme de construction
    cor.append(u"\\begin{enumerate}")
    cor.append(_(u"\\item On trace le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;") % (A, B, decimaux(AB)))
    cor.append(_(u"\\item On trace la demi-droite $[%s%s)$ en mesurant $\\widehat{%s%s%s}=%s\\degres$ ;") % (A, C, B, A, C, angBAC))
    cor.append(_(u"\\item On place le point $%s$ en mesurant $%s%s=\\unit[%s]{cm}$ ;") % (C, A, C, decimaux(AC)))
    cor.append(_(u"\\item On construit le point $%s$ en reportant au compas $%s%s=%s%s$ et $%s%s=%s%s$.")
               % (D, C, D, B, A, A, D, B, C))
    cor.append("\\end{enumerate}\n")
    # Figure
    cor.append(u"\\begin{pspicture}(%.3f,-1)(%.3f,%.3f)" % (min(0, x_C - AB) - 0.4, max(AB, x_C) + 0.4, y_C + 1))
    cor.append(u"\\pstGeonode[PosAngle={-135,-45,45,135}](0,0){%s}(%.3f,0){%s}(%.3f,%.3f){%s}(%.3f,%.3f){%s}" % (A, AB, B, x_C, y_C, C, x_D, y_D, D))
    cor.append(u"\\pstLineAB{%s}{%s}\\pstLineAB{%s}{%s}\\pstLineAB{%s}{%s}\\pstLineAB{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}"
               % (A, D, C, D, A, B, B, C, A, C))
    # Construction
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[C_1]" % (A, C))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[C_2]" % (A, C))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{C_2}{C_1}" % (A))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[D_1]" % (A, D))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[D_2]" % (A, D))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{D_2}{D_1}" % (A))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[D_3]" % (C, D))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[D_4]" % (C, D))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{D_4}{D_3}" % (C))
    # Codage
    cor.append(cotation_h((0, 0), (x_C, y_C), decimaux(AC), couleur="enonce"))
    cor.append(cotation((0, 0), (AB, 0), decimaux(AB), couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}" % (B, A, C, angBAC))
    cor.append(u"\\pstLineAB[linestyle=none]{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHashh}" % (A, D))
    cor.append(u"\\pstLineAB[linestyle=none]{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHashh}" % (B, C))
    cor.append(u"\\pstLineAB[linestyle=none]{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHash}" % (A, B))
    cor.append(u"\\pstLineAB[linestyle=none]{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHash}" % (D, C))
    cor.append(u"\\end{pspicture}")

def quest_parallelogramme_DDA(exo, cor):
    """On donne un parallélogramme avec la longueur des deux diagonales et l'angle entre les diagonales"""
    # choix d'angle : on pourrait donner l'angle BAC
    # diagonale ou demi-diagonale
    A, B, C, D, E = geo.choix_points(5)
    nom = shuffle_nom([A, B, C, D])
    BD = 0.2 * random.randint(20, 40)  # AB mesure entre 4cm et 8cm, tracé horizontalement
    AC = 0.2 * random.randint(20, 40)  # AC mesure entre 4cm et 8cm, tracé horizontalement
    angAEB = random.randint(35, 145)
    AE = AC // 2
    BE = BD // 2

    # calcul pour tracer
    AB = math.sqrt(AE ** 2 + BE ** 2 - 2 * AE * BE * cos(angAEB))
    angBAC = math.degrees(math.asin(BE * sin(angAEB) / AB))
    x_C = AC * cos(angBAC)
    y_D = y_C = round(AC * sin(angBAC), 4)
    x_D = round(x_C - AB, 4)

    exo.append(_(u"\\item Tracer un parallélogramme $%s$ de centre $%s$ tel que $%s%s=\\unit[%s]{cm}$, $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, E, A, C, decimaux(AC), B, D, decimaux(BD), A, E, B, angAEB))
    cor.append(_(u"\\item Tracer un parallélogramme $%s$ de centre $%s$ tel que $%s%s=\\unit[%s]{cm}$, $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, E, A, C, decimaux(AC), B, D, decimaux(BD), A, E, B, angAEB))
    cor.append(u"\\begin{enumerate}")
    cor.append(_(u"\\item On trace le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;") % (A, C, decimaux(AC)))
    cor.append(_(u"\\item Dans un parallélogramme les diagonales se coupent en leur milieu donc $%s%s=%s%s=\\unit[%s]{cm}$ et $%s%s=%s%s=\\unit[%s]{cm}$ ;")
               % (A, E, C, E, decimaux(AC // 2), B, E, E, D, decimaux(BD // 2)))
    cor.append("\\end{enumerate}\n")
    cor.append(u"\\begin{pspicture}(%.3f,-1)(%.3f,%.3f)" % (min(0, x_D) - 0.4, max(AB, x_C) + 0.4, y_C + 1))
    cor.append(u"\\pstGeonode[PosAngle={-135,-45,45,135,%s}](0,0){%s}(%.3f,0){%s}(%.3f,%.3f){%s}(%.3f,%.3f){%s}(%.3f,%.3f){%s}"
               % ([round(angBAC - (180 - angAEB) // 2), round(angBAC + angAEB // 2)][angAEB > 90], A, AB, B, x_C, y_C, C, x_D, y_D, D, x_C // 2, y_C // 2, E))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHashh}" % (A, E))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHashh}" % (C, E))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkCros}" % (B, E))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkCros}" % (D, E))
    cor.append(u"\\pstLineAB{%s}{%s}\\pstLineAB{%s}{%s}\\pstLineAB{%s}{%s}\\pstLineAB{%s}{%s}"
               % (A, D, C, D, A, B, B, C))
    cor.append(cotation_h((0, 0), (x_C // 2, y_C // 2), decimaux(AC // 2), couleur="enonce"))
    cor.append(cotation_h((x_C // 2, y_C // 2), (AB, 0), decimaux(BD // 2), couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}" % (A, E, B, angAEB))
    cor.append(u"\\end{pspicture}")

###################### LOSANGES ########################

def quest_losange_DD(exo, cor):
    """On donne un losange avec la longueur des deux diagonales"""
    # diagonale ou demi-diagonale
    A, B, C, D, E = geo.choix_points(5)
    nom = shuffle_nom([A, B, C, D])
    BD = 0.2 * random.randint(15, 25) # AB mesure entre 4cm et 8cm, tracé horizontalement
    AC = 0.2 * random.randint(20, 40)  # AC mesure entre 4cm et 8cm, tracé horizontalement

    exo.append(_(u"\\item Tracer un losange $%s$  tel que $%s%s=\\unit[%s]{cm}$ et $%s%s=\\unit[%s]{cm}$.\\par")
               % (nom, A, C, decimaux(AC), B, D, decimaux(BD)))
    cor.append(_(u"\\item Tracer un losange $%s$  tel que $%s%s=\\unit[%s]{cm}$ et $%s%s=\\unit[%s]{cm}$.\\par")
               % (nom, A, C, decimaux(AC), B, D, decimaux(BD)))

    cor.append(_(u"On note $%s$ le centre du losange.\\par") % E)

    cor.append("\\figureadroite{")
    cor.append(u"\\begin{pspicture}(%.3f,%.3f)(%.3f,%.3f)" % (-BD // 2 - 0.7, -AC // 2 - 0.4, BD // 2 + 0.4, AC // 2 + 0.4))
    cor.append(u"\\pstGeonode[PosAngle={-90,0,90,180}](0,%.3f){%s}(%.3f,0){%s}(0,%.3f){%s}(%.3f,0){%s}"
               % (-AC // 2, A, BD // 2, B, AC // 2, C, -BD // 2, D))
    cor.append("\\pstGeonode[PosAngle=-45](0,0){%s}" % E)

    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkCross}" % (A, E))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkCross}" % (C, E))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkCros}" % (B, E))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkCros}" % (D, E))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (A, B))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (C, D))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (B, C))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (D, A))
    cor.append("\\pstRightAngle[linecolor=calcul]{%s}{%s}{%s}" % (B, E, C))
    cor.append(cotation_h((-BD // 2, 0), (0, 0), decimaux(BD // 2), couleur="enonce"))
    cor.append(cotation_h((0, -AC // 2), (0, 0), decimaux(AC // 2), couleur="enonce"))
    cor.append(u"\\end{pspicture}}{")
    cor.append(_(u"Les diagonales du losange se coupent perpendiculairement en leur milieu~$%s$ ;  on a donc :") % E)
    cor.append(u"\\begin{enumerate}")
    cor.append(u"\\item $%s%s=%s%s=\\unit[%s]{cm}$ \\item $%s%s=%s%s=\\unit[%s]{cm}$ ;"
               % (A, E, C, E, decimaux(AC // 2), B, E, E, D, decimaux(BD // 2)))
    cor.append(u"\\item $(%s%s)\\perp(%s%s)$." % (A, C, B, D))
    cor.append("\\end{enumerate}}\n")

def quest_losange_CC(exo, cor):
    """On donne un losange avec la longueur d'un côté et un angle entre côtés"""
    # diagonale ou demi-diagonale
    A, B, C, D = geo.choix_points(4)
    nom = shuffle_nom([A, B, C, D])
    AB = 0.2 * random.randint(15, 25)  # AB mesure entre 4cm et 8cm, tracé horizontalement
    angBAD = random.randint(30, 150)
    # Calcul pour tracer
    x_D = AB * cos(angBAD)
    y_D = y_C = AB * sin(angBAD)
    x_C = x_D + AB

    exo.append(_(u"\\item Tracer un losange $%s$  tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, A, B, decimaux(AB), B, A, D, angBAD))
    cor.append(_(u"\\item Tracer un losange $%s$  tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, A, B, decimaux(AB), B, A, D, angBAD))

    cor.append(_(u"Les quatre côtés du losange sont de même longueur donc $%s%s=%s%s=%s%s=%s%s=\\unit[%s]{cm}$ ;") % (A, B, B, C, C, D, D, A, decimaux(AB)))
    cor.append(u"\\begin{enumerate}")
    cor.append(_(u"\\item On trace le côté $[%s%s]$ puis on mesure l'angle $\\widehat{%s%s%s}=%s\\degres$ ;") % (A, B, B, A, D, angBAD))
    cor.append(_(u"\\item ensuite on reporte au compas les longueurs $%s%s$ et $%s%s$ pour construire le point $%s$.") % (C, D, B, C, C))
    cor.append("\\end{enumerate}\n")
    cor.append(u"\\begin{pspicture}(%.3f,%.3f)(%.3f,%.3f)" % (min(0, x_D) - 0.4, 0 - 0.4, max(AB, x_C), y_D + 0.4))
    cor.append(u"\\pstGeonode[PosAngle={%s,%s,%s,%s}](0,0){%s}(%.3f,0){%s}(%.3f,%.3f){%s}(%.3f,%.3f){%s}"
               % (angBAD // 2 - 180, angBAD // 2 - 90, angBAD // 2, angBAD // 2 + 90, A, AB, B, x_C, y_C, C, x_D, y_D, D))

    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (A, B))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (C, D))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (B, C))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (D, A))

    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[C_1]" % (D, C))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[C_2]" % (D, C))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{C_2}{C_1}" % (D))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[C_3]" % (B, C))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[C_4]" % (B, C))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{C_4}{C_3}" % (B))
    cor.append(cotation((0, 0), (AB, 0), decimaux(AB), couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}" % (B, A, D, angBAD))
    cor.append(u"\\end{pspicture}")

def quest_losange_CD(exo, cor):
    """On donne un losange avec la longueur d'un côté et la mesure d'un angle entre un côté et une diagonale"""

    A, B, C, D = geo.choix_points(4)
    nom = shuffle_nom([A, B, C, D])
    AC = 0.1 * random.randint(40, 60)  # AB mesure entre 4cm et 7cm, tracé horizontalement
    angBAC = random.randint(25, 75)
    # Calcul pour tracer
    x_D = AC // 2
    y_D = AC / 2 * tan(angBAC)

    exo.append(_(u"\\item Tracer un losange $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, A, C, decimaux(AC), B, A, C, angBAC))
    cor.append(_(u"\\item Tracer un losange $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, A, C, decimaux(AC), B, A, C, angBAC))
    # Rédaction
    cor.append(_(u" Comme $%s$ est un losange, on sait que $\\widehat{%s%s%s}=\\widehat{%s%s%s}=\\widehat{%s%s%s}=\\widehat{%s%s%s}=%s\\degres$.")
               % (nom, B, A, C, A, C, B, A, C, D, C, A, D, angBAC))
    # Programme de construction
    cor.append(u"\\begin{enumerate}")
    cor.append(_(u"\\item On trace le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;") % (A, C, decimaux(AC)))
    cor.append(_(u"\\item On trace $\\widehat{%s%s%s}$ et $\\widehat{%s%s%s}$ pour construire le point $%s$ ;") % (B, A, C, A, C, B, B))
    cor.append(_(u"\\item On trace $\\widehat{%s%s%s}$ et $\\widehat{%s%s%s}$ pour construire le point $%s$ ;") % (A, C, D, C, A, D, D))
    cor.append("\\end{enumerate}\n")
    cor.append(u"\\begin{pspicture}(-0.4,%.3f)(%.3f,%.3f)" % (-y_D - 1, AC + 0.4, y_D + 1))
    # figure
    cor.append(u"\\pstGeonode[PosAngle={-180,0,90,-90}](0,0){%s}(%.3f,0){%s}(%.3f,%.3f){%s}(%.3f,%.3f){%s}" % (A, AC, C, x_D, y_D, D, x_D, -y_D, B))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (A, B))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (C, D))
    cor.append(u"\\pstLineAB[nodesepA=-1]{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (B, C))
    cor.append(u"\\pstLineAB[nodesepA=-1]{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (D, A))
    cor.append(u"\\pstLineAB{%s}{%s}" % (A, C))
    cor.append(cotation_h((0, 0), (AC, 0), decimaux(AC), couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce,Mark=MarkHash]{%s}{%s}{%s}{%s\\degres}" % (B, A, C, angBAC))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=calcul,Mark=MarkHash]{%s}{%s}{%s}{}" % (C, A, D))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=calcul,Mark=MarkHash]{%s}{%s}{%s}{}" % (D, C, A))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=calcul,Mark=MarkHash]{%s}{%s}{%s}{}" % (A, C, B))
    cor.append(u"\\end{pspicture}")

def quest_losange_CDbis(exo, cor):
    """On donne un losange avec la longueur d'un côté et la mesure d'un angle entre un côté et une diagonale"""

    A, B, C, D = geo.choix_points(4)
    nom = shuffle_nom([A, B, C, D])
    AC = 0.1 * random.randint(40, 60)  # AB mesure entre 4cm et 7cm, tracé horizontalement
    angCDA = 2 * random.randint(15, 70)
    # Calcul pour tracer
    angCAD = (180 - angCDA) // 2
    x_D = AC // 2
    y_D = AC / 2 * tan(angCAD)

    exo.append(_(u"\\item Tracer un losange $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, A, C, decimaux(AC), C, D, A, angCDA))
    cor.append(_(u"\\item Tracer un losange $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par")
               % (nom, A, C, decimaux(AC), C, D, A, angCDA))

    # Rédaction des calculs
    cor.append(_(u"Les quatre côtés du losange sont de même longueur donc $%s%s=%s%s=%s%s=%s%s$.\\par") % (A, B, B, C, C, D, D, A))
    cor.append(_(u"Ainsi, le triangle $%s%s%s$ est isocèle en $%s$ et on peut calculer la mesure des angles $\\widehat{%s%s%s}=\\widehat{%s%s%s}$.\\par")
               % (A, C, D, A, A, C, D, C, A, D))
    cor.append(_(u"Dans un triangle, la somme des angles du triangle est égale à 180\\degres\\\\"))
    cor.append(_(u"donc $\\widehat{%s%s%s}=\\widehat{%s%s%s}=(180\\degres-%s)\\div2=%s\\degres$") % (A, C, D, C, A, D, angCDA, angCAD))
    # Programme de construction
    cor.append(u"\\begin{enumerate}")
    cor.append(_(u"\\item On trace le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;") % (A, C, decimaux(AC)))
    cor.append(_(u"\\item On trace $\\widehat{%s%s%s}$ et $\\widehat{%s%s%s}$ pour construire le point $%s$ ;") % (B, A, C, A, C, B, B))
    cor.append(_(u"\\item On trace $\\widehat{%s%s%s}$ et $\\widehat{%s%s%s}$ pour construire le point $%s$ ;") % (A, C, D, C, A, D, D))
    cor.append("\\end{enumerate}\n")

    cor.append(u"\\begin{pspicture}(-0.4,%.3f)(%.3f,%.3f)" % (-y_D - 1, AC + 0.4, y_D + 1))
    # Figure
    cor.append(u"\\pstGeonode[PosAngle={-180,0,90,-90}](0,0){%s}(%.3f,0){%s}(%.3f,%.3f){%s}(%.3f,%.3f){%s}" % (A, AC, C, x_D, y_D, D, x_D, -y_D, B))
    cor.append(u"\\pstLineAB{%s}{%s}" % (A, C))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}" % (A, B))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}" % (C, B))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}" % (C, D))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}" % (A, D))
    # codage
    cor.append(u"\\color{calcul}\\pstMarkAngle[linecolor=calcul,Mark=MarkHash]{%s}{%s}{%s}{}" % (B, A, C))
    cor.append(u"\\color{calcul}\\pstMarkAngle[linecolor=calcul,Mark=MarkHash]{%s}{%s}{%s}{%s\\degres}" % (C, A, D, angCAD))
    cor.append(u"\\color{calcul}\\pstMarkAngle[linecolor=calcul,Mark=MarkHash]{%s}{%s}{%s}{%s\\degres}" % (D, C, A, angCAD))
    cor.append(u"\\color{calcul}\\pstMarkAngle[linecolor=calcul,Mark=MarkHash]{%s}{%s}{%s}{}" % (A, C, B))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (A, B))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (C, B))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (C, D))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (A, D))
    cor.append(cotation((0, 0), (AC, 0), decimaux(AC), couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}" % (A, D, C, angCDA))
    cor.append(u"\\end{pspicture}")

################### CARRÉ #######################

def carre_diag(exo, cor):
    """trace un carré dont on donne la longueur de la diagonale"""

    A, B, C, D, E = geo.choix_points(5)
    nom = shuffle_nom([A, B, C, D])
    BD = AC = 0.2 * random.randint(20, 40)  # AC mesure entre 4cm et 8cm, tracé horizontalement

    exo.append(_(u"\\item Tracer un carré $%s$  tel que $%s%s=\\unit[%s]{cm}$.\\par")
               % (nom, A, C, decimaux(AC)))
    cor.append(_(u"\\item Tracer un carré $%s$  tel que $%s%s=\\unit[%s]{cm}$.\\par")
               % (nom, A, C, decimaux(AC)))

    cor.append(_(u"On note $%s$ le centre du carré.\\par") % (E))
    cor.append(_(u" Les diagonales du carré se coupent perpendiculairement en leur milieu $%s$ donc on a :") % E)
    cor.append("\\begin{enumerate}")
    cor.append(u"\\item $(%s%s)\\perp(%s%s)$." % (A, C, B, D))
    cor.append(u"\\item  $%s%s=%s%s=%s%s=%s%s=\\unit[%s]{cm}$ ;"
               % (A, E, C, E, B, E, D, E, decimaux(BD // 2)))
    cor.append("\\end{enumerate}\n")

    cor.append("\\begin{pspicture}(%.3f,%.3f)(%.3f,%.3f)" % (-BD / 2 - 0.4, -AC / 2 - 0.4, BD / 2 + 0.4, AC / 2 + 0.4))
    cor.append("\\pstGeonode[PosAngle={-90,0,90,180}](0,%.3f){%s}(%.3f,0){%s}(0,%.3f){%s}(%.3f,0){%s}"
               % (-AC // 2, A, BD // 2, B, AC // 2, C, -BD // 2, D))
    cor.append("\\pstGeonode[PosAngle=-45](0,0){%s}" % E)

    cor.append("\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkCros}" % (A, E))
    cor.append("\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkCros}" % (C, E))
    cor.append("\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkCros}" % (B, E))
    cor.append("\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkCros}" % (D, E))
    cor.append("\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (A, B))
    cor.append("\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (C, D))
    cor.append("\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (B, C))
    cor.append("\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}" % (D, A))
    cor.append("\\pstRightAngle[linecolor=calcul]{%s}{%s}{%s}" % (B, E, C))
    cor.append(cotation_h((-BD // 2, 0), (0, 0), decimaux(BD // 2), couleur="enonce"))
    cor.append(u"\\end{pspicture}")



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

'''
Created on 19 déc. 2014

@author: jerome
'''
from pyromaths.outils.Arithmetique import carrerise
from pyromaths.outils.decimaux import decimaux
from pyromaths.outils import Priorites3
from pyromaths import classes

from math import sqrt
class SquareRoot:
    '''
    Définit la classe SquareRoot permettant de manipuler des racines carrées.
    
    SquareRoot([a,b], [c, d], e) ou SquareRoot([a,b], [c, d], [e, None]) permet de définir a*sqrt(b)+c*sqrt(d)+e
    
    Cette définition permet d'utiliser pyromaths.outils.Priorites3
    
    >>> from pyromaths.classes.SquareRoot import SquareRoot
    >>> repr(SquareRoot(-4,[-2,1],[3,45],[-1,7],8))
    SquareRoot([[-4, None], [-2, 1], [3, 45], [-1, 7], [8, None]])
    '''


    def __init__(self, *radicandes):
        '''
        Constructor
        '''
        #  print radicandes, len(radicandes), radicandes[0], len(radicandes[0])
        if len(radicandes) == 1 and len(radicandes[0]) == 2 and not isinstance(radicandes[0][0], list) and not isinstance(radicandes[0][1], list):
            # SquareRoot([3, 4]),
            self.racines = [radicandes[0]]
        else:
            if len(radicandes) == 1 and (len(radicandes[0]) != 2 or isinstance(radicandes[0][0], list) or isinstance(radicandes[0][1], list)):
                # SquareRoot([1, [3, 4]])
                radicandes = radicandes[0]
            self.racines = []
            for arg in radicandes:
                if isinstance(arg, list) and len(arg) == 2:
                    # On gère a*sqrt(b)
                    if isinstance(arg[1], (float, int))and arg[1] < 0:
                        raise ValueError(u'Le radicande doit être un nombre positif.')
                    else:
                        self.racines.append(arg)
                elif isinstance(arg, (float, int)):
                    self.racines.append([arg, None])
                else:
                    raise ValueError(u'Not Implemented : SquareRoot(%s)' % arg)

    def __str__(self):
        r"""Renvoie une version LaTeX d'un objet SquareRoot.

        >>> from pyromaths.classes.SquareRoot import SquareRoot
        >>> str(SquareRoot([[-4, None], [-2, 1], [3, 45], [-1, 7], [8, None]]))
        -4-2\,\sqrt{1}+3\,\sqrt{45}-\sqrt{7}+8

        :rtype: String
        """
        def print_coef(coef):
            """Gère le format du coef
            """
            if isinstance(coef, (float, int)):
                if coef > 0: return "+" + decimaux(coef)
                else: return decimaux(coef)
            if isinstance(coef, classes.Fractions.Fraction):
                if isinstance(coef.n, int) and isinstance(coef.d, int) and coef.n < 0 and coef.d > 0:
                    return "-" + str(Fraction(-coef.n, coef.d, coef.code))
                return "+" + str(coef)
            if isinstance(coef, str):
                texte = "(" + "".join(Priorites3.texify([Priorites3.splitting(coef)])) + ")"
                if texte[0] != "-": return "+" + texte
                else: return texte
        s = ""
        for m in self.racines:
            if m[1] == None:
                # pas de racine ici
                s = s + print_coef(m[0])
            else:
                # Racine carrée
                if m[0] == 1:
                    s += r'+\sqrt{%s}' % m[1]
                elif m[0] == -1:
                    s += r'-\sqrt{%s}' % m[1]
                else:
                    s += print_coef(m[0]) + r'\,\sqrt{%s}' % m[1]
        # supprime le + en début de séquence
        s = s.lstrip("+")
        if not s: s = "0"
        return s

    def __repr__(self):
        """Renvoie une chaîne de caractère représentant un :mod:`SquareRoot`
        évaluable pour créer un :mod:`SquareRoot`.

        >>> from pyromaths.classes.SquareRoot import SquareRoot
        >>> repr(SquareRoot(-4, [-2, 1], [3, 45], [-1, 7], 8))
        SquareRoot([[-4, None], [-2, 1], [3, 45], [-1, 7], [8, None]])

        :rtype: String
        """
        return "SquareRoot(%s)" % self.racines

    def __len__(self):
        """*object*\ .\ **__len__**\ ()

        Renvoie le nombre d'éléments de l'objet SquareRoot.

        >>> from pyromaths.classes.SquareRoot import SquareRoot
        >>> len(SquareRoot(3, [2, 2], [4, 5]))
        3

        :rtype: integer
        """
        return len(self.racines)

    def __getitem__(self, i):
        """*object*\ .\ **__getitem__**\ (*integer*)

        Renvoie le i ème élément de l'objet SquareRoot.

        >>> from pyromaths.classes.SquareRoot import SquareRoot
        >>> SquareRoot(3, [2, 2], [4, 5])[2]
        [4, 5]

        :rtype: list
        """
        return self.racines[i]


    def __add__(self, other):
        """Renvoie la somme d'un objet SquareRoot et d'un nombre.
        
        >>> from pyromaths.classes.SquareRoot import SquareRoot
        >>> repr(SquareRoot([3,45],3)+SquareRoot([2,45]))
        SquareRoot([[3, None], [5, 45]])
        
        :rtype: SquareRoot
        """

        if not isinstance(other, SquareRoot):
            other = SquareRoot([other, None])
        self.racines.extend(other.racines)
        return SquareRoot(self.racines).simplifie()

    def __radd__(self, other):
        """
        >>> from pyromaths.classes.SquareRoot import SquareRoot
        >>> repr(2+SquareRoot([3,45],3))
        SquareRoot([[5, None], [3, 45]])

        :rtype: SquareRoot
        """
        other = SquareRoot([other, None])
        other.racines.extend(self.racines)
        return SquareRoot(other.racines).simplifie()

    def __neg__(self):
        """*object*\ .\ **__neg__**\ ()

        ``p.__neg__()`` est équivalent à ``-p`` est équivalent à ``p = -p``

        Renvoie l'opposé d'un objet SquareRoot.

        :rtype: SquareRoot
        """
        if self.EstDecomposable() or self.EstReductible(): return '-%r' % self.simplifie()
        r = list(self.racines)
        for i in range(len(r)):
            r[i][0] = -r[i][0]
        return SquareRoot(r)

    def __abs__(self):
        """ Renvoie la valeur absolue d'un objet SquareRoot
        
        >>> from pyromaths.classes.SquareRoot import SquareRoot
        >>> repr(abs(SquareRoot([5, 5], [-2, 7])))
        SquareRoot([[5, 5], [-2, 7]])
        >>> repr(abs(SquareRoot([-5, 5], [2, 7])))
        SquareRoot([[5, 5], [-2, 7]])
       
        :rtype: SquareRoot
        """
        t = 0
        for e in self.racines:
            if e[1] == None: t += e[0]
            else: t += e[0] * sqrt(e[1])
        if t > 0: return self
        else: return -self

    def evaluate(self, other):
        t, t2 = 0, 0
        for e in self.racines:
            if e[1] == None:
                t += e[0]
            else:
                t += e[0] * sqrt(e[1])
        if isinstance(other, SquareRoot):
            for e in other.racines:
                if e[1] == None:
                    t2 += e[0]
                else:
                    t2 += e[0] * sqrt(e[1])
        else:
            t2=other
        return t, t2

    def __lt__(self, other):
        t, t2 = self.evaluate(other)
        if t<t2: return True
        else: return False

    def __le__(self, other):
        t, t2 = self.evaluate(other)
        if t<=t2: return True
        else: return False

    def __eq__(self, other):
        t, t2 = self.evaluate(other)
        if t==t2: return True
        else: return False

    def __ne__(self, other):
        t, t2 = self.evaluate(other)
        if t!=t2: return True
        else: return False

    def __gt__(self, other):
        t, t2 = self.evaluate(other)
        if t>t2: return True
        else: return False

    def __ge__(self, other):
        t, t2 = self.evaluate(other)
        if t>=t2: return True
        else: return False

    def __mul__(self, other):
        """Multiplie un objet SquareRoot par un nombre.
        
        >>> from pyromaths.classes.SquareRoot import SquareRoot
        >>> repr(SquareRoot([3,45],3)*SquareRoot([2,45],-1))
        SquareRoot([['6*45', None], [-3, 45], [6, 45], [-3, None]])
        """
        if not isinstance(other, SquareRoot):
            other = SquareRoot([other, None])
        reduction = False
        if self.EstReductible():
            self = self.simplifie()
            reduction = True
        if other.EstReductible():
            other = other.simplifie()
            reduction = True
        if reduction: return '%r*%r' % (self, other)
        lprod = []
        for e in self.racines:
            for f in other.racines:
                if e[1] == None or f[1] == None:
                    lprod.append([e[0] * f[0], e[1] or f[1]])
                elif e[1] == f[1]:
                    lprod.append(['%r*%r' % (e[0] * f[0], e[1]), None])
                elif carrerise(e[1]) == 1 or carrerise(f[1]) == 1:
                    if carrerise(e[1]) == 1: e[0], e[1] = e[0] * int(sqrt(e[1])), 1
                    if carrerise(f[1]) == 1: f[0], f[1] = f[0] * int(sqrt(f[1])), 1
                    lprod.append([e[0] * f[0], e[1] * f[1]])
                else:
                    lprod.append([e[0] * f[0], e[1] * f[1]])
        return SquareRoot(lprod)

    def __rmul__(self, other):
        """
        >>> from pyromaths.classes.SquareRoot import SquareRoot
        >>> repr(5*SquareRoot([3,45],3))
        SquareRoot([[15, 45], [15, None]])

        :rtype: SquareRoot
        """
        return SquareRoot([other, None]) * self

    def  __floordiv__(self, other):
        """
        Division entière par un entier
        
        >>> from pyromaths.classes.SquareRoot import SquareRoot
        >>> repr(SquareRoot([10, 8], [15, 5])//5)
        SquareRoot([[2, 8], [3, 5]])

        :rtype: SquareRoot
        """
        if isinstance(other, int):
            r = list(self.racines)
            for i in range(len(r)):
                r[i][0] = r[i][0] // other
            return SquareRoot(r)
        else:
            raise NotImplemented
    def EstDecomposable(self):
        """
        Renvoie True si une des racines est de la forme sqrt{a**2*b} avec a != 1
        
        >>> from pyromaths.classes.SquareRoot import SquareRoot
        >>> SquareRoot([5, 8], [1, 7]).EstDecomposable()
        True
        >>> SquareRoot([5, 7], [1, 7]).EstDecomposable()
        False
     
        :rtype: Boolean
        """
        for e in self.racines:
            if e[1] != None and (carrerise(e[1]) != e[1] or e[1] == 1):
                return True
        return False

    def EstReductible(self):
        """
        Renvoie True si la somme de racines est réductible

        >>> from pyromaths.classes.SquareRoot import SquareRoot
        >>> SquareRoot([5, 8], [1, 45]).EstReductible()
        False
        >>> SquareRoot([5, 8], [1, 8]).EstReductible()
        True

        :rtype: Boolean
        """
        lradicandes = []
        rationnel = False
        for e in self.racines:
            if e[1] != None:
                if e[1] in lradicandes:
                    return True
                else:
                    lradicandes.append(e[1])
            elif e[1] == None:
                if rationnel: return True
                else: rationnel = True
        return False

    def Decompose(self):
        """
        Décompose une unique racine carrée de la forme a*sqrt(b^2*c) en a*sqrt(b^2)*sqrt(c)
        
        >>> from pyromaths.classes.SquareRoot import SquareRoot
        >>> SquareRoot([5, 8]).Decompose()
        SquareRoot([[5, 4]])*SquareRoot([[1, 2]])

        :rtype: string
        """
        racine = self.racines[0]
        if racine[1] == None: return repr(racine[0])
        if isinstance(racine[1], int):
            complement = carrerise(racine[1])
            if complement == 1:
                if racine[0] == 1:
                    return int(sqrt(racine[1]))
                if racine[0] == -1:
                    return -int(sqrt(racine[1]))
                if racine[1] == 1:
                    return str(racine[0])
                return '%r*%r' % (racine[0], int(sqrt(racine[1])))
            if complement == racine[1]:
                return repr(self)
            return '%r*%r' % (SquareRoot([racine[0], racine[1] // complement]), SquareRoot([1, complement]))
        raise ValueError(u'Not Implemented : SquareRoot(%s)' % racine)

    def simplifie(self):
        """
        Additionne les nombres rationnels et décompose les racines carrées.
         
        >>> from pyromaths.classes.SquareRoot import SquareRoot
        >>> repr(SquareRoot([[3, 9]]).simplifie())
        '3*3'
        >>> repr(SquareRoot(-2, [-2, 1], [3, 1], [-1, 7], 8).simplifie())
        SquareRoot([[6, None], [1, 1], [-1, 7]])
        >>> repr(SquareRoot(-2, [-2, 1], [3, 45], [-1, 7], 8).simplifie())
        SquareRoot([[6, None], [-2, 1], [3, 45], [-1, 7]])
        >>> repr(SquareRoot([-2,1],[3,45]).simplifie())
        '-2+SquareRoot([[3, 9]])*SquareRoot([[1, 5]])'
        
        :rtype: SquareRoot ou String
        """
        decomposable = self.EstDecomposable()
        reductible = self.EstReductible()
        if not decomposable and not reductible:
            return self
        if reductible:
            racines = []
            # racines = [[sum([x[0] for i, x in enumerate(self.racines) if x[1] == None]), None]]
            # TODO: cas de fractions
            lsomme = [x[0] for i, x in enumerate(self.racines) if x[1] == None]
            s = ''
            for ls in lsomme:
                if isinstance(ls, str): s += '+%s' % ls
                else: s += '+%r' % ls
            s.lstrip('+')
            racines.append([eval(s), None])
            for pos in reversed([i for i, x in enumerate(self.racines) if x[1] == None]):
                    self.racines.__delitem__(pos)
            if racines[0][0] == 0: racines = []
            while len(self.racines) > 0:
                # racines.append([sum([x[0] for i, x in enumerate(self.racines) if x[1] == self.racines[0][1]]), self.racines[0][1]])
                # TODO: cas de fractions
                lsomme = [x[0] for i, x in enumerate(self.racines) if x[1] == self.racines[0][1]]
                s = ''
                for ls in lsomme:
                    if isinstance(ls, str): s += '+%s' % ls
                    else: s += '+%r' % ls
                s.lstrip('+')
                racines.append([eval(s), self.racines[0][1]])
                if racines[-1][0] == 0: racines.pop(-1)
                for pos in reversed([i for i, x in enumerate(self.racines) if x[1] == self.racines[0][1]]):
                    self.racines.__delitem__(pos)
            if racines:
                if len(racines) == 1 and racines[0][1] == None:
                    return racines[0][0]
                else:
                    return SquareRoot(racines)
            else: return 0
        # Décomposable
        s = ''
        if len(self.racines) == 1 and isinstance(SquareRoot(self.racines).Decompose(), int):
            return SquareRoot(self.racines).Decompose()
        for e in self.racines:
            decomposee = SquareRoot(e).Decompose()
            if isinstance(decomposee, int): decomposee = str(decomposee)
            if decomposee[0]in'+-':
                s += decomposee
            else:
                s += '+' + decomposee
        s = s.lstrip('+')
        return s

    def __float__(self):
        value=0
        for item in self.racines:
            if item[1]!=None:
                if type(item[0])==str:
                    value = value + float(eval(item[0])) * sqrt(item[1])
                else:
                    value = value + item[0]*sqrt(item[1])
            else:
                value = value + item[0]
        return value



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
import random
from decimal import Decimal
from math import acos, asin, atan, pi, sin, cos, tan

from fractions import Fraction
from pyromaths.outils.decimaux import decimaux
from pyromaths.outils.Geometrie import couples_pythagore, choix_points
from pyromaths.ex import LegacyExercise


#
# ------------------- THEOREME DE PYTHAGORE -------------------
def fig_tr_rect(lg):
    # renvoie les angles au centre des trois sommets du triangle ABC rectange en C
    a = random.randrange(360)
    if a < 180:
        b = a + 180
    else:
        b = a - 180
    c = float(Fraction(int((180 - 2 * acos(lg[1] * 1 / lg[2]) * 180 / pi) * 100), 100) + a)
    if c < 0:
        c = c + 360
    return (str(a), str(b), str(c))


def enonce_pythagore(noms, angles, longueurs, cotes, nom_tr, long0,
                     long1, diam=0):
    if diam:
        return (
            noms[0],
            angles[0],
            angles[0],
            noms[1],
            angles[1],
            angles[1],
            noms[2],
            angles[2],
            angles[2],
            int(float(angles[0]) - 90),
            cotes[2],
            noms[2],
            cotes[long0],
            decimaux(longueurs[long0], 1),
            cotes[long1],
            decimaux(longueurs[long1], 1),
            cotes[(3 - long0) - long1],
        )
    else:
        return (
            nom_tr,
            noms[2],
            cotes[long0],
            decimaux(longueurs[long0], 1),
            cotes[long1],
            decimaux(longueurs[long1], 1),
            cotes[(3 - long0) - long1],
        )


def _exo_pythagore():
    types_exercice = [[2, random.randrange(2)], [0, 1]]
    random.shuffle(types_exercice)
    random.shuffle(types_exercice[0])
    random.shuffle(types_exercice[1])
    exo = ["\\exercice\n\\begin{multicols}{2}\n  \\begin{enumerate}"]
    cor = ["\\exercice*\n\\begin{multicols}{2}\n  \\begin{enumerate}"]
    for j in range(2):
        while True:
            longueurs = couples_pythagore[random.randrange(len(couples_pythagore))]
            longueurs = [longueurs[i] / 10 for i in range(3)]
            if inegalite_triangulaire(longueurs):
                break
        noms = choix_points(3)
        angles = fig_tr_rect(longueurs)
        nom_tr = nom_triangle(noms)
        long0, long1 = types_exercice[j]
        cotes = cotes_sommets(noms)
        enonce = \
            """    \\item Soit $%s$ un triangle rectangle en $%s$ tel que :\\par
$%s=\\unit[%s]{cm}$ et $%s=\\unit[%s]{cm}$.\\par
Calculer la longueur $%s$.""" % \
            enonce_pythagore(noms, angles, longueurs, cotes, nom_tr, long0, long1)
        exo.append(enonce)
        cor.append(enonce)
        cor.append("\\par\\dotfill{}\\par\n")
        cor.append(u"Le triangle $%s$ est rectangle en $%s$.\\par" % \
                   (nom_tr, noms[2]))
        cor.append(u"Son hypoténuse est $[%s]$.\\par" % (cotes[2]))
        cor.append(u"D'après le \\textbf{théorème de Pythagore} :")
        cor.append("\\[%s^2=%s^2+%s^2\\]" % (cotes[2], cotes[0], cotes[1]))
        if long0 == 2 or long1 == 2:
            cor.append("\\[%s^2=%s^2-%s^2\\kern1cm\\text{(On cherche }%s)\\]" %
                       (cotes[(3 - long0) - long1], cotes[2], cotes[((4 -
                                                                      long0) - long1) % 2], cotes[(3 - long0) - long1]))
        if long0 == 2 or long1 == 2:
            cor.append("\\[%s^2=%s^2-%s^2\\]" % (cotes[(3 - long0) - long1],
                                                 decimaux(longueurs[2], 1),
                                                 decimaux(longueurs[((4 - long0) - long1) % 2], 1)))
        else:
            cor.append("\\[%s^2=%s^2+%s^2\\]" % (cotes[2],
                decimaux(longueurs[0], 1),
                decimaux(longueurs[1], 1)))
        if long0 == 2 or long1 == 2:
            cor.append("\\[%s^2=%s-%s\\]" % (cotes[(3 - long0) - long1],
                decimaux(longueurs[2] ** 2, 1),
                decimaux(longueurs[((4 - long0) - long1) % 2] ** 2, 1)))
        else:
            cor.append("\\[%s^2=%s+%s\\]" % (cotes[2],
                decimaux(longueurs[0] ** 2),
                decimaux(longueurs[1] ** 2, 1)))
        if long0 == 2 or long1 == 2:
            cor.append("\\[%s^2=%s\\]" % (cotes[(3 - long0) - long1],
                decimaux(longueurs[2] ** 2 - longueurs[((4 - long0) - long1) % 2] ** 2, 1)))
        else:
            cor.append("\\[%s^2=%s\\]" % (cotes[2],
                decimaux(longueurs[0] ** 2 + longueurs[1] ** 2, 1)))
        if long0 == 2 or long1 == 2:
            cor.append("\\[ \\boxed{\\text{Donc }%s=\\sqrt{%s}=\\unit[%s]{cm}}\\]" %
                       (cotes[(3 - long0) - long1],
                           decimaux(longueurs[2] ** 2 - longueurs[((4 - long0) - long1) % 2] ** 2, 1),
                           decimaux(longueurs[(3 - long0) - long1], 1)))
        else:
            cor.append("\\[\\boxed{\\text{Donc }%s=\\sqrt{%s}=\\unit[%s]{cm}}\\]" %
                       (cotes[2],
                           decimaux(longueurs[0] ** 2 + longueurs[1] ** 2, 1),
                           decimaux(longueurs[2], 1)))
        if j == 0:
            exo.append("\\columnbreak")
            cor.append("\\columnbreak")
    exo.append("\\end{enumerate}\n\\end{multicols}\n")
    cor.append("\\end{enumerate}\n\\end{multicols}\n")
    return (exo, cor)

class exo_pythagore(LegacyExercise):
    """Théorème de Pythagore"""

    tags = ["Quatrième"]
    function = _exo_pythagore


def nom_triangle(noms):  # renvoie le nom du triangle dans un ordre aleatoire
    a = random.randrange(3)
    b = (random.randrange(2) + 1 + a) % 3
    c = (3 - a) - b
    return '%s%s%s' % (noms[a], noms[b], noms[c])


def cotes_sommets(noms):  # renvoie les noms des 3 cotes du triangle en finissant par l'hypotenuse
    return (noms[1] + noms[2], noms[0] + noms[2], noms[0] + noms[1])


#
# ------------------- CERCLE ET THEOREME DE PYTHAGORE -------------------
#


def _exo_triangle_cercle():
    exo = ["\\exercice"]
    cor = ["\\exercice*"]
    while True:
        longueurs = couples_pythagore[random.randrange(len(couples_pythagore))]
        longueurs = [longueurs[i] / 10 for i in range(3)]
        if inegalite_triangulaire(longueurs):
            break
    noms = choix_points(3)
    angles = fig_tr_rect(longueurs)
    nom_tr = nom_triangle(noms)
    long0 = random.randrange(3)
    long1 = (random.randrange(2) + 1 + long0) % 3
    cotes = cotes_sommets(noms)
    enonce = \
        u"""\\begin{minipage}{4cm}
\\begin{pspicture}(-2,-2)(2,2)
\\SpecialCoor\\psset{PointSymbol=none}
\\pstGeonode[PointName=%s,PosAngle=%s](1.5;%s){a}
\\pstGeonode[PointName=%s,PosAngle=%s](1.5;%s){b}
\\pstGeonode[PointName=%s,PosAngle=%s](1.5;%s){c}
\\pspolygon(a)(b)(c)\\pscircle(0,0){1.5}
\\rput(1.8;%s){$\\big(\\mathcal{C}\\big)$}
\\end{pspicture}
\\end{minipage}\\hfill
\\begin{minipage}{13cm}
$\\big(\\mathcal{C}\\big)$ est un cercle de diamètre $[%s]$ et $%s$ est un point de $\\big(\\mathcal{C}\\big)$.\\par
On donne $%s=\\unit[%s]{cm}$ et $%s=\\unit[%s]{cm}$.\\par
Calculer la longueur $%s$.""" % \
        enonce_pythagore(noms, angles, longueurs, cotes, nom_tr, long0, long1, diam=1)
    exo.append(enonce)
    cor.append(enonce)
    cor.append("\\par\\dotfill{}\\\\\n")
    cor.append(u"$[%s]$ est le diamètre du cercle circonscrit au triangle $%s$.\\par" %
               (cotes[2], nom_tr))
    cor.append("\\fbox{Donc le triangle %s est rectangle en %s.}\\\\\n" %
               (nom_tr, noms[2]))
    cor.append(u"D'après le \\textbf{théorème de Pythagore} :")
    cor.append(u"\\[%s^2=%s^2+%s^2\\kern1cm\\text{(car }[%s]\\text{ est \\emph{l'hypoténuse})}\\]" %
               (cotes[2], cotes[0], cotes[1], cotes[2]))
    if long0 == 2 or long1 == 2:
        cor.append("\\[%s^2=%s^2-%s^2\\kern1cm\\text{(On cherche }%s)\\]" %
                   (cotes[(3 - long0) - long1], cotes[2], cotes[((4 -
                                                                  long0) - long1) % 2], cotes[(3 - long0) - long1]))
    if long0 == 2 or long1 == 2:
        cor.append("\\[%s^2=%s^2-%s^2\\]" % (cotes[(3 - long0) - long1],
                                             decimaux(longueurs[2], 1), decimaux(longueurs[((4 - long0) -
                                                                                     long1) % 2], 1)))
    else:
        cor.append("\\[%s^2=%s^2+%s^2\\]" % (cotes[2], decimaux(longueurs[0], 1),
                                             decimaux(longueurs[1], 1)))
    if long0 == 2 or long1 == 2:
        cor.append("\\[%s^2=%s-%s\\]" % (cotes[(3 - long0) - long1],
                                         decimaux(longueurs[2] ** 2, 1), decimaux(longueurs[((4 -
                                                                                       long0) - long1) % 2] ** 2, 1)))
    else:
        cor.append("\\[%s^2=%s+%s\\]" % (cotes[2], decimaux(longueurs[0] **
                                                          2, 1), decimaux(longueurs[1] ** 2, 1)))
    if long0 == 2 or long1 == 2:
        cor.append("\\[%s^2=%s\\]" % (cotes[(3 - long0) - long1],
                                      decimaux(longueurs[2] ** 2 - longueurs[((4 - long0) -
                                                                            long1) % 2] ** 2, 1)))
    else:
        cor.append("\\[%s^2=%s\\]" % (cotes[2], decimaux(longueurs[0] **
                                                       2 + longueurs[1] ** 2, 1)))
    if long0 == 2 or long1 == 2:
        cor.append("\\[\\boxed{\\text{Donc }%s=\\sqrt{%s}=\\unit[%s]{cm}}\\]" %
                   (cotes[(3 - long0) - long1],
                       decimaux(longueurs[2] ** 2 - longueurs[((4 - long0) - long1) % 2] ** 2, 1),
                       decimaux(longueurs[(3 - long0) - long1], 1)))
    else:
        cor.append("\\[\\boxed{\\text{Donc }%s=\\sqrt{%s}=\\unit[%s]{cm}}\\]" %
                   (cotes[2],
                       decimaux(longueurs[0] ** 2 + longueurs[1] ** 2, 1),
                       decimaux(longueurs[2], 1)))
    exo.append("\\end{minipage}\n")
    cor.append("\\end{minipage}\n")
    return (exo, cor)

class exo_triangle_cercle(LegacyExercise):
    """Cercle et théorème de Pythagore"""

    tags = ["Quatrième"]
    function = _exo_triangle_cercle


#
# ------------------- RECIPROQUE DU THEOREME DE PYTHAGORE -------------------
#


def _exo_reciproque_pythagore():
    exo = ["\\exercice"]
    cor = ["\\exercice*"]
    while True:
        longueurs = couples_pythagore[random.randrange(len(couples_pythagore))]
        longueurs = [longueurs[i] / 10 for i in range(3)]
        if inegalite_triangulaire(longueurs):
            break
    noms = choix_points(3)
    nom_tr = nom_triangle(noms)
    l = [i for i in range(3)]
    n = [l.pop(random.randrange(3 - i)) for i in range(3)]
    c = cotes_sommets(noms)
    recip = (nom_tr, c[n[0]], decimaux(longueurs[n[0]], 1), c[n[1]], decimaux(longueurs[n[1]], 1),
             c[n[2]], decimaux(longueurs[n[2]], 1), nom_tr)
    enonce = \
        """Soit $%s$ un triangle tel que : $\\quad %s=\\unit[%s]{cm}\\quad$, $\\quad %s=\\unit[%s]{cm}\\quad$ et $\\quad %s=\\unit[%s]{cm}$.\\par
Quelle est la nature du triangle $%s$?
""" % \
        recip
    exo.append(enonce)
    cor.append(enonce)
    cor.append("\\par\\dotfill{}\\\\\n")
    cor.append(u"Le triangle %s n'est ni isocèle, ni équilatéral.\\par\n" %
               nom_tr)
    cor.append("$\\left.")
    cor.append("\\renewcommand{\\arraystretch}{2}")
    cor.append("\\begin{array}{l}")

    cor.append(u"\\bullet %s^2=%s^2=%s\\qquad\\text{(}[%s]\\text{ est le plus grand côté.)}\\\\\n" %
               (c[2], decimaux(longueurs[2], 1), decimaux(longueurs[2] ** 2, 1), c[2]))
    cor.append("\\bullet  %s^2+%s^2=%s^2+%s^2=%s \n" % (c[0], c[1],
                                                        decimaux(longueurs[0], 1), decimaux(longueurs[1], 1),
                                                        decimaux(longueurs[0] **
                                                               2 + longueurs[1] ** 2, 1)))
    cor.append("\\end{array}")
    cor.append("\\right\\rbrace$")
    cor.append(u"""Donc $%s^2=%s^2+%s^2$.\\par
D'après la \\textbf{réciproque du théorème de Pythagore},
\\fbox{le triangle $%s$ est rectangle en $%s$.}""" %
               (c[2], c[0], c[1], nom_tr, noms[2]))
    return (exo, cor)

class exo_reciproque_pythagore(LegacyExercise):
    """Réciproque du théorème de Pythagore"""

    tags = ["Quatrième"]
    function = _exo_reciproque_pythagore


#
# ------------------- THEOREME DE THALES -------------------
#


def valeurs_thales(pyromax):
    liste = [0, 0, 0, 0, 0, 0, 0, 0]
    while liste == [0, 0, 0, 0, 0, 0, 0, 0]:
        for i in range(3):
            liste[i] = random.randrange(2)
    a = random.randrange(liste.count(1))
    for i in range(3):
        if liste[i]:
            if not a:
                liste[i + 3] = 1
            a = a - 1
        else:
            liste[i + 3] = 1  # on doit connaitre le numeratuer ou le denominateur
    for i in range(2):  # AB et AE  ou  AB et BE  ou  AE et EB
        if liste[i] and liste[i + 3]:  # i est le rapport complet. On choisit une des 3 formes ci-dessus
            a = random.randrange(2)
            liste[i + 6] = 1
            liste[i + 3 * a] = 0
            rapport = [i + 3 * ((a + 1) % 3), i + 3 * ((a + 2) % 3)]
            rapport.sort()
    if liste[2] and liste[5]:
        rapport = [2, 5]
    valeurs = [0, 0, 0, 0, 0, 0, 0, 0]
    for i in range(3):
        if liste[i]:
            valeurs[i] = random.randrange(15, pyromax)
        if liste[i + 3] and liste[i]:
            valeurs[i + 3] = random.randrange(5, valeurs[i] - 9)
        elif liste[i + 3]:
            valeurs[i + 3] = random.randrange(5, pyromax)
    if liste[6]:
        valeurs[6] = random.randrange(5, pyromax)
    if liste[7]:
        valeurs[7] = random.randrange(5, pyromax)

    # --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
    # type_thales=valeur_alea(-1,1) # -1 si papillon, 1 si triangle

    type_thales = 1
    valeurs.append((rapport, type_thales))
    if test_valeurs_thales(valeurs, rapport, type_thales):
        return valeurs
    else:
        return 0


def test_valeurs_thales(valeurs, rapport, type_thales):
    v = [valeurs[i] for i in range(8)]
    if rapport[0] // 3 == 0 and rapport[1] // 3 == 2:  # On donne AB et EB
        v[rapport[0] + 3] = (v[rapport[0]] - v[rapport[1]]) * type_thales
    elif rapport[0] // 3 == 1 and rapport[1] // 3 == 2:  # On donne AE et EB
        v[rapport[0] - 3] = v[rapport[0]] * type_thales + v[rapport[1]]
    if v[rapport[0] % 3]:  # rapport est AE/AB
        rapp = Fraction(v[rapport[0] % 3 + 3], v[rapport[0] % 3])
    else:
        rapp = 0
    for i in range(3):
        if not v[i] and rapp:
            v[i] = v[i + 3] / rapp
        elif not v[i + 3]:
            v[i + 3] = v[i] * rapp
    if inegalite_triangulaire(v[0:3]) and inegalite_triangulaire(v[3:6]) and .3 < rapp < .7:
        return v
    else:
        return 0


def inegalite_triangulaire(a):  # renvoie 1 si c'est un triangle, 0 sinon
    vrai = 0
    coef = Fraction(12, 10)  # evite les triangles trop ecrases
    if a[0] > a[1] and a[0] > a[2]:
        if a[1] + a[2] > coef * a[0]:
            vrai = 1
    elif a[1] > a[0] and a[1] > a[2]:
        if a[0] + a[2] > coef * a[1]:
            vrai = 1
    elif a[2] > a[0] and a[2] > a[1]:
        if a[0] + a[1] > coef * a[2]:
            vrai = 1
    return vrai


def _exo_thales():
    exo = ["\\exercice"]
    cor = ["\\exercice*"]
    noms = choix_points(5)  # les noms des sommets
    arrondi = random.randrange(1, 4)
    text_arrondi = ['dix', 'cent', 'mill'][arrondi - 1] + u'ième'
    while True:
        valeurs = valeurs_thales(70)  # les longueurs en mm
        if valeurs:
            break
    exo.append(tex_fig_thales(noms, valeurs))
    exo.append(tex_enonce_thales(noms, valeurs, text_arrondi))
    cor.append(tex_fig_thales(noms, valeurs))
    cor.append(tex_enonce_thales(noms, valeurs, text_arrondi))
    cor.append(tex_resolution_thales0(noms, valeurs))
    cor.append(tex_resolution_thales1(noms, valeurs))
    cor.append(tex_resolution_thales2(noms, valeurs))
    cor.append(tex_resolution_thales3(noms, valeurs, arrondi))
    return (exo, cor)

class exo_thales(LegacyExercise):
    """Théorème de Thalès"""

    tags = ["Quatrième"]
    function = _exo_thales


def long_val(noms,
             valeurs):  # renvoie un tuple contenant les noms des segments et leur longueur puis les noms des longueurs a calculer
    liste = []
    for i in range(8):
        if valeurs[i]:
            liste.append(creer_noms(noms, i))
            liste.append(decimaux(valeurs[i], 1))
    for i in range(6):
        if not valeurs[i] and valeurs[8][0][0] % 3 != i % 3:
            liste.append(creer_noms(noms, i))
    return liste


def lAB(a):  # renvoie AB
    return str(a[0]) + str(a[1])


def creer_noms(noms, i):
    if i == 0:
        return str(noms[0]) + str(noms[1])
    elif i == 1:
        return str(noms[0]) + str(noms[2])
    elif i == 2:
        return str(noms[1]) + str(noms[2])
    elif i == 3:
        return str(noms[0]) + str(noms[3])
    elif i == 4:
        return str(noms[0]) + str(noms[4])
    elif i == 5:
        return str(noms[3]) + str(noms[4])
    elif i == 6:
        return str(noms[3]) + str(noms[1])
    elif i == 7:
        return str(noms[4]) + str(noms[2])


def tex_enonce_thales(noms, valeurs, arrondi):
    texte = \
        u'{Sur la figure ci-contre, les droites $(%s)$ et $(%s)$ sont parallèles.\\par\n' % \
        (lAB(noms[1:3]), lAB(noms[3:5]))
    liste = long_val(noms, valeurs)
    texte = texte + \
            'On donne $%s=\\unit[%s]{cm}$,$\\quad %s=\\unit[%s]{cm}$, $\\quad %s=\\unit[%s]{cm}\\quad$ et $\\quad %s~=~\\unit[%s]{cm}$.\\par\n' % \
            tuple(liste[0:8])
    texte = texte + 'Calculer $%s$ et $%s$, ' % tuple(liste[8:10])
    texte = texte + 'arrondies au %s}\n' % arrondi
    return texte


def tex_resolution_thales0(n, v):
    return u"""Dans le triangle $%s$,~ $%s$ est sur le côté $[%s]$,~
$%s$ est sur le côté $[%s]$ et les droites $(%s)$ et $(%s)$ sont
parallèles.\\par
D'après le \\textbf{théorème de Thalès} :
$\\qquad\\mathbf{\\cfrac{%s}{%s}=\\cfrac{%s}{%s}=\\cfrac{%s}{%s}}$""" % \
           (
               n[0] + n[1] + n[2],
               n[3],
               n[0] + n[1],
               n[4],
               n[0] + n[2],
               n[1] + n[2],
               n[3] + n[4],
               creer_noms(n, 0),
               creer_noms(n, 3),
               creer_noms(n, 1),
               creer_noms(n, 4),
               creer_noms(n, 2),
               creer_noms(n, 5),
           )


def tex_resolution_thales1(n, v):
    r = v[8][0][0] % 3  # grand rapport
    if v[8][1] == 1:
        sgn = '+'
    else:
        sgn = '-'
    if v[r] and v[r + 3]:  # on connait les deux rapports
        donnees = 0
    elif v[r + 3]:

        # on connait  le petit rapport, mais pas le grand

        v[r] = v[r + 6] + v[r + 3] * v[8][1]
        donnees = (creer_noms(n, r), creer_noms(n, r + 6), sgn,
                   creer_noms(n, r + 3), decimaux(v[r], 1))
    else:
        v[r + 3] = (v[r] - v[r + 6]) * v[8][1]
        if sgn == '+':
            donnees = (creer_noms(n, r + 3), creer_noms(n, r), '-',
                       creer_noms(n, r + 6), decimaux(v[r + 3], 1))
        else:
            donnees = (creer_noms(n, r + 3), creer_noms(n, r + 6), '-',
                       creer_noms(n, r), decimaux(v[r + 3], 1))
    if donnees:
        print(donnees)
        return '\\vspace{1ex}\\par De plus $%s=%s%s%s=\\unit[%s]{cm}$\n' % donnees
    else:
        return ''


def tex_resolution_thales2(n, v):
    donnees = []
    for i in range(3):
        if v[i]:
            donnees.append(decimaux(v[i], 1))
        else:
            donnees.append(creer_noms(n, i))
        if v[i + 3]:
            donnees.append(decimaux(v[i + 3], 1))
        else:
            donnees.append(creer_noms(n, i + 3))
    return '\\[\\frac{%s}{%s}=\\frac{%s}{%s}=\\frac{%s}{%s}\\]' % tuple(donnees)


def nom_ou_valeur(n, v, i):
    if v[i]:
        return decimaux(v[i], 1)
    else:
        return creer_noms(n, i)


def valeur_exacte(a, approx=3, unit=1):
    nb = decimaux(a, 1)
    if unit:
        if nb.count(',') and (len(nb) - nb.find(',')) - 1 > approx:
            return '\\simeq\\unit[' + decimaux(round(a, approx), 1) + ']{cm}'
        else:
            return '=\\unit[' + decimaux(a, 1) + ']{cm}'
    else:
        if nb.count(',') and (len(nb) - nb.find(',')) - 1 > approx:
            return '\\simeq' + decimaux(round(a, approx), 1)
        else:
            return '=' + decimaux(a, 1)


def tex_resolution_thales3(n, v, arrondi):
    r = v[8][0][0] % 3  # grand rapport
    donnees = []
    for i in range(3):
        if i != r:
            donnees.extend([nom_ou_valeur(n, v, r), nom_ou_valeur(n, v, r + 3), nom_ou_valeur(n, v, i),
                            nom_ou_valeur(n, v, i + 3)])
            if v[i]:  # on cherche i+3
                donnees.extend([creer_noms(n, i + 3), decimaux(v[i], 1), decimaux(v[r + 3], 1), decimaux(v[r], 1),
                                valeur_exacte(v[i] * v[r + 3] / v[r], approx=arrondi)])
            else:
                donnees.extend([creer_noms(n, i), decimaux(v[i + 3], 1), decimaux(v[r], 1), decimaux(v[r + 3], 1),
                                valeur_exacte(v[r] * v[i + 3] / v[r + 3], approx=arrondi)])
    texte = \
        '$\\cfrac{%s}{%s}=\\cfrac{%s}{%s}\\quad$ donc $\\quad \\boxed{%s=\\cfrac{%s\\times %s}{%s}%s}$\\par\n ' % tuple(
            donnees[0:9])
    texte = texte + \
            '$\\cfrac{%s}{%s}=\\cfrac{%s}{%s}\\quad$ donc $\\quad\\boxed{%s=\\cfrac{%s\\times %s}{%s}%s}$\\par\n' % tuple(
        donnees[9:18])
    return texte


# def pyromax(a):
#    maxi = a[0]
#    for i in xrange(len(a)):
#        if a[i] > maxi:
#            maxi = a[i]
#    return maxi
#
#
# def pyromin(a):
#    mini = a[0]
#    for i in xrange(len(a)):
#        if a[i] < mini:
#            mini = a[i]
#    return mini


def fig_thales(noms, valeurs):
    v = test_valeurs_thales(valeurs[0:8], valeurs[8][0], valeurs[8][1])
    type_thales = valeurs[8][1]
    angle = Decimal(int(100 * acos((v[0] ** 2 + v[1] ** 2 - v[2] ** 2) / (2 * v[0] * v[1])) * 180 / pi)) / 100
    v = [int(v[i] * 100) / 100 for i in range(8)]
    mini_x = int(100 * min(0, v[1] * cos(float(angle) * pi / 180), v[3] * type_thales,
                           (v[4] * cos(float(angle) * pi / 180)) * type_thales)) / 100 - 1.5
    mini_y = int(100 * min(0, v[4] * sin(float(angle) * pi / 180) * type_thales)) / 100 - 1.5
    maxi_x = int(100 * max(v[0], v[1] * cos(float(angle) * pi / 180))) / 100 + 1.5
    maxi_y = int(100 * v[1] * sin(float(angle) * pi / 180)) / 100 + .5
    echelle = int(400 / max(abs(mini_x) + maxi_x, abs(mini_y) + maxi_y)) / 100
    if type_thales == 1:
        return (
            echelle,
            mini_x,
            mini_y,
            maxi_x,
            maxi_y,
            225,
            angle + 45,
            noms[0],
            noms[1],
            noms[2],
            v[0],
            v[1],
            angle,
            - 45,
            angle + 90,
            noms[3],
            noms[4],
            v[3],
            v[4],
            angle,
        )
    else:
        return (
            echelle,
            mini_x,
            mini_y,
            maxi_x,
            maxi_y,
            135,
            angle + 45,
            noms[0],
            noms[1],
            noms[2],
            v[0],
            v[1],
            angle,
            135,
            angle + 180,
            noms[3],
            noms[4],
            - v[3],
            - v[4],
            angle,
        )


def tex_fig_thales(noms, valeurs):
    donnees = fig_thales(noms, valeurs)
    enonce = \
        '''\\figureadroite{
  \\psset{PointSymbol=none,unit=%s}
  \\begin{pspicture}(%s,%s)(%s,%s)
    \\SpecialCoor
    \\pstTriangle[PosAngleA=%s,PosAngleB=-45,PosAngleC=%s,PointNameA=%s,
      PointNameB=%s,PointNameC=%s](0,0){a}(%s,0){b}(%s;%s){c}
    \\pstTriangle[PosAngleB=%s,PosAngleC=%s,PointSymbolA=none,
      PointName=none,PointNameB=%s,PointNameC=%s](0,0){a}(%s,0){b}(%s;%s){c}
  \\end{pspicture}
}''' % \
        donnees
    return enonce


#
# ------------------- TRIGONOMETRIE -------------------
#


def _exo_trigo():
    exo = ["\\exercice"]
    cor = ["\\exercice*"]
    s = choix_points(6)
    n1 = cotes_sommets(s[0:3])
    n2 = cotes_sommets(s[3:6])
    v = valeurs_trigo()
    (l1, l2) = enonce_trigo(((s[0:3], n1, v[0]), (s[3:6], n2, v[1])))
    exo.extend(l1)
    cor.extend(l2)
    return (exo, cor)

class exo_trigo(LegacyExercise):
    """Cosinus d\'un angle aigu"""

    tags = ["Quatrième"]
    function = _exo_trigo


def enonce_trigo(v):
    (exo, cor) = ([], [])
    (l, lt) = ([], [])
    for j in range(2):
        f = (('\\sin', 1, 0), ('\\cos', 2, 0), ('\\tan', 1, 2))[v[j][2][0]]
        for i in range(2):
            l.append(v[j][1][f[i + 1]])
            l.append(v[j][2][i + 1])
        l.append(angle(v[j][0], 1))
        l.append(v[j][2][3])
    for j in range(2):
        tmp = []
        for i in range(3):
            if len(l[2 * i + 6 * j]) < 3:
                if l[2 * i + 6 * j + 1]:
                    lt.append('$%s=\\unit[%s]{cm}$' % (l[2 * i + 6 * j],
                                                       decimaux(l[2 * i + 6 * j + 1], 1)))
                else:
                    tmp = 'la longueur $%s$' % l[2 * i + 6 * j]
            elif l[2 * i + 6 * j + 1]:
                lt.append('$%s=%s\\degres$' % (l[2 * i + 6 * j], l[2 * i +
                                                                   6 * j + 1]))
            else:
                lt.append('la mesure de l\'angle $%s$' % l[2 * i + 6 * j])
        if tmp:
            lt.append(tmp)
    exo.append('\\begin{multicols}{2}')
    exo.append('\\begin{enumerate}')
    cor.append('\\begin{multicols}{2}')
    cor.append('\\begin{enumerate}')
    arrondi = random.randrange(1, 4)
    text_arrondi = ['dix', 'cent', 'mill'][arrondi - 1] + u'ième'
    tr = nom_triangle(v[0][0])
    exo.append('\\item $%s$ est un triangle rectangle en $%s$ tel que :\\par' %
               (tr, v[0][0][0]))
    exo.append('%s et %s.\\par\nCalculer %s, arrondie au %s.\\par' % tuple(lt[0:3] + [text_arrondi]))
    cor.append('\\item $%s$ est un triangle rectangle en $%s$ tel que :\\par' %
               (tr, v[0][0][0]))
    cor.append('%s et %s.\\par\nCalculer %s, arrondie au %s.\\par' % tuple(lt[0:3] + [text_arrondi]))
    cor.append('Dans le triangle $%s$ rectangle en $%s$,' % (tr, v[0][0][0]))  # résolution
    v2 = (v[0][1], v[0][2])
    l2 = l[0:6]
    cor.extend(resolution_trigo(v2, l2, arrondi))
    tr = nom_triangle(v[1][0])
    exo.append('\\columnbreak')
    cor.append('\\columnbreak')
    arrondi = random.randrange(1, 4)
    text_arrondi = ['dix', 'cent', 'mill'][arrondi - 1] + u'ième'
    exo.append('\\item $%s$ est un triangle rectangle en $%s$ tel que :\\par' %
               (tr, v[1][0][0]))
    exo.append('''%s et %s.\\par
Calculer %s, arrondie au %s.\\par''' %
               tuple(lt[3:6] + [text_arrondi]))
    cor.append('\\item $%s$ est un triangle rectangle en $%s$ tel que :\\par' %
               (tr, v[1][0][0]))
    cor.append('%s et %s.\\par\nCalculer %s, arrondie au %s.\\par' % tuple(lt[3:6] + [text_arrondi]))
    #    cor.append("""\\dotfill{}\\par\\vspace{2ex}")
    cor.append('Dans le triangle $%s$ rectangle en $%s$,' % (tr, v[1][0][0]))  # résolution
    v2 = (v[1][1], v[1][2])
    l2 = l[6:12]
    cor.extend(resolution_trigo(v2, l2, arrondi))
    exo.append('\\end{enumerate}')
    exo.append('\\end{multicols}')
    cor.append('\\end{enumerate}')
    cor.append('\\end{multicols}')
    return (exo, cor)


def resolution_trigo(v2, l2, arrondi):
    cor = []
    f = (('\\sin', 1, 0), ('\\cos', 2, 0), ('\\tan', 1, 2))[v2[1][0]]
    cor.append('\\[ %s%s=\\cfrac{%s}{%s} \\]' % (f[0], l2[4], v2[0][f[1]],
                                                 v2[0][f[2]]))
    if not v2[1][3]:
        cor.append('\\[ %s%s=\\cfrac{%s}{%s} \\]' % (f[0], l2[4],
                                                     decimaux(v2[1][1], 1), decimaux(v2[1][2], 1)))
        if f[0] == '\\sin':
            r = asin(v2[1][1] / v2[1][2]) * 180 / pi
        elif f[0] == '\\cos':
            r = acos(v2[1][1] / v2[1][2]) * 180 / pi
        else:
            r = atan(v2[1][1] / v2[1][2]) * 180 / pi
        cor.append(r'\[ \boxed{%s=%s^{-1}\left(\cfrac{%s}{%s}\right) %s\degres} \]' %
                   (l2[4], f[0], decimaux(v2[1][1], 1), decimaux(v2[1][2], 1),
                    valeur_exacte(r, approx=arrondi, unit=0)))
    elif not v2[1][1]:
        cor.append('\\[ %s%s=\\cfrac{%s}{%s} \\]' % (f[0], v2[1][3],
                                                     v2[0][f[1]], decimaux(v2[1][2], 1)))
        if f[0] == '\\sin':
            r = sin(v2[1][3] * pi / 180)
        elif f[0] == '\\cos':
            r = cos(v2[1][3] * pi / 180)
        else:
            r = tan(v2[1][3] * pi / 180)
        r = r * v2[1][2]
        cor.append(r'\[ \boxed{%s=%s%s\times %s %s} \]' %
                   (v2[0][f[1]], f[0], v2[1][3], decimaux(v2[1][2], 1),
                    valeur_exacte(r, approx=arrondi)))
    else:
        cor.append('\\[ %s%s=\\cfrac{%s}{%s} \\]' % (f[0], v2[1][3],
                                                     decimaux(v2[1][1], 1), v2[0][f[2]]))
        if f[0] == '\\sin':
            r = sin(v2[1][3] * pi / 180)
        elif f[0] == '\\cos':
            r = cos(v2[1][3] * pi / 180)
        else:
            r = tan(v2[1][3] * pi / 180)
        r = v2[1][1] / r
        cor.append(r'\[ \boxed{%s=\cfrac{%s}{%s%s} %s} \]' %
                   (v2[0][f[2]], decimaux(v2[1][1], 1), f[0], v2[1][3],
                    valeur_exacte(r, approx=arrondi)))
    return cor


def angle(s,
          n):  # renvoie \\widehat{ABC} où s est la liste des 3 sommets du triangle et n est le rang du sommet de l'angle dans cette liste
    return '\\widehat{%s%s%s}' % (s[(n + 2) % 3], s[n], s[(n + 1) % 3])


def valeurs_trigo():
    l = [random.randrange(10, 121) / 10 for dummy in range(3)]
    l.sort()
    l.append(random.randrange(15, 76))
    trigo = 1
    if random.randrange(2):  # on choisit en 1er une longueur et un angle
        if random.randrange(2):  # on connait la première des deux longueurs
            v = (trigo, l[0], 0, l[3])
        else:
            v = (trigo, 0, l[0], l[3])
        v = (v, (trigo, l[1], l[2], 0))
    else:

        # on choisit en 1er deux longueurs

        v = (trigo, l[1], l[2], 0)
        if random.randrange(2):  # on connait la première des deux longueurs
            v = (v, (trigo, l[0], 0, l[3]))
        else:
            v = (v, (trigo, 0, l[0], l[3]))
    return v

# TODO: utils/pyromaths generate exo_thales:1 plante sur l'arcos (ligne 605)



from pyromaths.outils.Arithmetique import *

def exo_second_degre():
    """Créer une exercice comprenant des équations et inéquations du second degré. PAS FINI."""
    exo = _("\\exercice\n\
           \\begin{multicols}{2}\n\
           \\noindent " + u"Question 1 : Chercher les racines des polynômes suivants :\\par\n")
    cor = _("\\exercice*\n\
           \\begin{multicols}{2}\n\ \\noindent " + u"Question 1 : Chercher les racines des polynômes suivants :\\par\n")

    # # Equations
    pol1 = choix_coeffs(False, 2, True)
    pol2 = choix_coeffs(False, 1, False)
    pol3 = choix_coeffs(False, 0, True)
    pol4 = choix_coeffs(False, 2, False)

    # # Inequations
    pol5 = choix_coeffs(True, 0, True)
    pol6 = choix_coeffs(False, 2, True)
    pol7 = choix_coeffs(False, 1, False)
    pol8 = choix_coeffs(False, 2, False)

    # # Somme et produit des racines
    pol9 = choix_coeffs(True, 2, False)

    # # Problème du rectangle
    pol10 = choix_coeffs(True, 2, False)

    exo += "\\par a)$" + pol1[1].print_signe("=") + "$\n"
    exo += "\\par b)$" + pol2[1].print_signe("=") + "$\n"
    exo += "\\par c)$" + pol3[1].print_signe("=") + "$\n"
    exo += "\\par d)$" + pol4[1].print_signe("=") + "$\n"

    cor += "\\par a)$" + pol1[1].print_signe("=") + "$\n"
    cor += "\\par b)$" + pol2[1].print_signe("=") + "$\n"
    cor += "\\par c)$" + pol3[1].print_signe("=") + "$\n"
    cor += "\\par d)$" + pol4[1].print_signe("=") + "$\n"

    cor += _("\\par a) On calcule le discriminant : $\\Delta = ") + str(pol1[1].delta) + "$.\n"
    cor += _("\\par Comme le discrimimant est $") + pol1[1].signedelta + _(u"$, on en déduit que ce polynôme possède $") + str(pol1[1].nbrac) + _("$ racines.")

    exo += _(u"\\par Question 2 : Résoudre les inéquations suivantes : ")
    cor += _(u"\\par Question 2 : Résoudre les inéquations suivantes : ")

    exo += "\\par a)$" + pol5[1].print_signe("\\leqslant{}") + "$\n"
    exo += "\\par b)$" + pol6[1].print_signe("\\geqslant{}") + "$\n"
    exo += "\\par c)$" + pol7[1].print_signe(">") + "$\n"
    exo += "\\par d)$" + pol8[1].print_signe("<") + "$\n"

    cor += "\\par a)$" + pol5[1].print_signe("\\leqslant{}") + "$\n"
    cor += "\\par b)$" + pol6[1].print_signe("\\geqslant{}") + "$\n"
    cor += "\\par c)$" + pol7[1].print_signe(">") + "$\n"
    cor += "\\par d)$" + pol8[1].print_signe("<") + "$\n"

    exo += _("\\par Question 3 : Trouver deux nombres sachant que leur somme est ") + str(pol9[1].b) + _(" et leur produit ") + str(pol9[1].c) + ".\n"
    cor += _("\\par Question 3 : Trouver deux nombres sachant que leur somme est ") + str(pol9[1].b) + _(" et leur produit ") + str(pol9[1].c) + ".\n"

    exo += _(u"\\par Question 4 : Trouver les dimensions d'un rectangle, sachant que son périmètre est ") + str(pol9[1].b) + _(" et son aire ") + str(pol9[1].c) + "."
    cor += _(u"\\par Question 4 : Trouver les dimensions d'un rectangle, sachant que son périmètre est ") + str(pol9[1].b) + _(" et son aire ") + str(pol9[1].c) + "."

    exo += "\\end{multicols}\n"
    cor += "\\end{multicols}\n"
    return (exo, cor)

if __name__ == "__main__":
    from TEST.imprimetest import *
    exo, cor = exo_second_degre()
    imprime_TeX(exo + "\\pagebreak" + cor, "secondegre.tex")
